/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.axis2.module;

import dk.sosi.seal.model.Message;
import dk.sosi.seal.model.constants.IDValues;
import dk.sosi.seal.model.constants.NameSpaces;
import dk.sosi.seal.model.constants.SOAPTags;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNode;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.util.XMLUtils;
import org.w3c.dom.Document;

import javax.xml.namespace.QName;
import java.util.Iterator;
import java.util.Map;

/**
 * @author Nymand
 */
public class Axis2Util {

    /**
     * @param doc
     * @param isFaultMessage
     * @throws Exception
     */
    private static void addHeadersToSOAPEnvelope(SOAPEnvelope envelope, Document doc, boolean isFaultMessage) throws Exception {
        
    	// Make sure that all namespaces are present in the axis response
        Map namespaces = isFaultMessage ? NameSpaces.SOSI_FAULT_NAMESPACES : NameSpaces.SOSI_NAMESPACES;

        Iterator it = namespaces.keySet().iterator();
        while (it.hasNext()) {
            String namespace = (String) it.next();
            envelope.declareNamespace((String) namespaces.get(namespace), namespace);
        }

        envelope.addAttribute(IDValues.id, SOAPTags.ENVELOPE_UNPREFIXED, null);

        // Now transfer the generated header elements back to the axis context
        OMElement omDoc = XMLUtils.toOM(doc.getDocumentElement());
        Iterator childElements = omDoc.getFirstChildWithName(new QName(NameSpaces.SOAP_SCHEMA, SOAPTags.HEADER_UNPREFIXED)).getChildElements();
        while (childElements.hasNext()) {
        	envelope.getHeader().addChild((OMNode) childElements.next());
        }
        
    }

    private static void replaceSOAPBody(SOAPEnvelope envelope, Document doc) throws Exception {
    	OMElement omDoc = XMLUtils.toOM(doc.getDocumentElement());
        OMElement element = omDoc.getFirstChildWithName(new QName(NameSpaces.SOAP_SCHEMA, SOAPTags.BODY_UNPREFIXED)).getFirstElement();
        envelope.getBody().getFirstElement().detach();
        envelope.getBody().addChild(element);
	}

    public static void addMessageToSOAPEnvelope(MessageContext messageContext, Message message) throws Exception {
    	SOAPEnvelope envelope = messageContext.getEnvelope();
        Document doc = message.serialize2DOMDocument();
		addHeadersToSOAPEnvelope(envelope, doc, message.isFault());
        if (message.isFault()) {
        	replaceSOAPBody(envelope, doc);
        }
    }

	public static String getParameterValue(MessageContext context, String parameter) {
        Parameter par = context.getConfigurationContext().getAxisConfiguration().getParameter(parameter);
        return par!= null ?  (String) par.getValue() : null;
    }

    public static Object getImplementation(MessageContext messageContext, String parameter) throws AxisFault {
        String clazz = getParameterValue(messageContext, parameter);
        if (clazz != null) {
            try {
                Class aClass = Class.forName(clazz);
                return aClass.newInstance();
            } catch (ClassNotFoundException e) {
                throw new AxisFault("Failed to get implementation :" + clazz + " e: " + e, e);
            } catch (IllegalAccessException e) {
                throw new AxisFault("Failed to get implementation :" + clazz + " e: " + e, e);
            } catch (InstantiationException e) {
                throw new AxisFault("Failed to get implementation :" + clazz + " e: " + e, e);
            }
        } else {
            throw new AxisFault("Failed to setup sosi handler, missing parameter in config: " + parameter);
        }
    }
}
