/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.axis2.module;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.ReplyHeader;
import dk.sosi.seal.model.Request;

/**
 * Interface to be implemented by clients using this module.
 *
 *
 * @since 1.5.6
 */
public interface SOSIClient {

	/**
	 * The key used to retrieve the idcard that the client application has set via the Options on the axis2 serviceclient
	 */
	String SOSI_IDCARD_KEY = "sosi:idcard";

    /**
     * Return an instance of the SOSI factory to be used for creating ID-cards authenticating the client.
     * @return
     *          instance of <code>SOSIFactory</code>.
     */
    SOSIFactory createSOSIFactory();

    /**
     * Create the SOSI request.
     * @param factory
     *          Instance of <code>SOSIFactory</code> to use for request generation.
     * @return SOSI request.
     */
    Request createRequest(SOSIFactory factory);

    /**
     * Authorize the reply received from an invoked service.
     * @param reply
     *      Reply that should be authorized.
     * @return
     *      true, if reply is authorized, false otherwise.
     */
    boolean isAuthorized(ReplyHeader replyHeader);

}
