/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.axis2.module;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.handlers.AbstractHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author Nymand
 */
public abstract class SOSIHandler extends AbstractHandler {
    protected final Log log = LogFactory.getLog(this.getClass());
    protected SOSIClient client;
    protected SOSIService service;
    private String mode = null;
    private static final String SOSI_MODE = "SOSIMode";

    private static final String SOSI_CLIENT = "SOSIClient";
    private static final String SOSI_SERVICE = "SOSIService";

    protected static final String PROP_SOSI_FACTORY = "sosi:factory";
    protected static final String PROP_SOSI_CURRENT_HEADER = "sosi:current_header";

    protected abstract void invokeClient(MessageContext messageContext) throws AxisFault;

    protected abstract void invokeService(MessageContext messageContext) throws AxisFault;

    public InvocationResponse invoke(MessageContext messageContext) throws AxisFault {
        log.info("SOSIHandler invoked.");
        setup(messageContext);

        if (client != null) {
            log.info("Invoking client: " + client);
            invokeClient(messageContext);
        } else {
            log.info("Invoking service: " + service);
            invokeService(messageContext);
        }

        return InvocationResponse.CONTINUE;
    }

    private void setup(MessageContext messageContext) throws AxisFault {
        if (mode == null) {
            mode = Axis2Util.getParameterValue(messageContext, SOSI_MODE);
            if (mode.equals("client")) {
                setupClient(messageContext);
            } else if (mode.equals("service")) {
                setupService(messageContext);
            } else {
                throw new AxisFault("Failed to setup SOSIHandler, bad value of parameter SOSIMode: " + mode);
            }
        }
    }

    private void setupClient(MessageContext messageContext) throws AxisFault {
        client = (SOSIClient) Axis2Util.getImplementation(messageContext, SOSI_CLIENT);
    }

    private void setupService(MessageContext messageContext) throws AxisFault {
        service = (SOSIService) Axis2Util.getImplementation(messageContext, SOSI_SERVICE);
    }
}
