/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.axis2.module;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.ReplyHeader;
import dk.sosi.seal.model.RequestHeader;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;

/**
 * @since 1.5.6
 */
public class SOSIReceiver extends SOSIHandler {

	protected void invokeClient(MessageContext messageContext) throws AxisFault {
        try {
            //Factory is already setup by sender
            SOSIFactory factory = (SOSIFactory) messageContext.getOperationContext().getProperty(PROP_SOSI_FACTORY);

            SOAPEnvelope envelope = messageContext.getOperationContext().getMessageContext("In").getEnvelope();
			if (messageContext.isFault()) {
            	String xml = envelope.toString();
            	// FaultCode and FaultString reside in the soap body, deserialize the whole reply!
            	Reply reply = factory.deserializeReply(xml);
            	log.info("Received error reply: " + reply.getFaultCode());
            } else {
            	String xml = envelope.getHeader().toString();
            	// No need to deserialize the entire reply ...
            	ReplyHeader header = factory.deserializeReplyHeader(xml);
            	log.info("Received reply for request with id: " + header.getRequestID());
            	client.isAuthorized(header);
            }

        } catch (RuntimeException e) {
            throw new AxisFault("Couldn't handle response in SOSIReciever");
        }
    }

    protected void invokeService(MessageContext messageContext) throws AxisFault {

        SOSIFactory factory = service.createSOSIFactory();
        messageContext.setProperty(PROP_SOSI_FACTORY, factory);

        SOAPHeader soapHeader = messageContext.getEnvelope().getHeader();
        RequestHeader header = factory.deserializeRequestHeader(soapHeader.toString());

        if (!service.isAuthorized(header.getIDCard())) {
        	throw new AxisFault("Request not authorized");
        }

        log.info("Request was authorized by service. MessageID for request: " + header.getMessageID());
        messageContext.setProperty(PROP_SOSI_CURRENT_HEADER, header);

    }


}
