/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.axis2.module;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.model.RequestHeader;
import org.apache.axiom.soap.SOAPFault;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;

/**
 * @author Nymand
 */
public class SOSISender extends SOSIHandler {

	/**
	 * Invoke the client sending the message.
	 *
	 * @param messageContext
	 * @throws AxisFault
	 */
	protected void invokeClient(MessageContext messageContext) throws AxisFault {
		SOSIFactory factory = client.createSOSIFactory();
		messageContext.getOperationContext().setProperty(PROP_SOSI_FACTORY, factory);

		// Get the IDCard
		IDCard idCard = (IDCard) messageContext.getOptions().getProperty(SOSIClient.SOSI_IDCARD_KEY);

		// Create Request
		Request request = client.createRequest(factory);
		request.setIDCard(idCard);

		try {

			Axis2Util.addMessageToSOAPEnvelope(messageContext, request);
			log.info("sending: " + messageContext.getEnvelope());

		} catch (Throwable e) {
			log.error(e);
			throw new AxisFault(e.getMessage(), e);
		}
	}

	/**
	 * Invoke the service, sending the message.
	 *
	 * @param messageContext
	 * @throws AxisFault
	 */
	protected void invokeService(MessageContext messageContext) throws AxisFault {
		// Factory has been created by receiver
		MessageContext inMessageContext = messageContext.getOperationContext().getMessageContext("In");
		SOSIFactory factory = (SOSIFactory) inMessageContext.getProperty(PROP_SOSI_FACTORY);

		RequestHeader requestHeader = (RequestHeader) inMessageContext.getProperty(PROP_SOSI_CURRENT_HEADER);
		log.info("In messageId: " + requestHeader.getMessageID());

		Reply reply;

		if (messageContext.isFault()) {
			SOAPFault fault = messageContext.getEnvelope().getBody().getFault();
			reply = service.createErrorReply(factory, requestHeader, fault.getReason().getText(), fault.getReason().getText());
		} else {
			reply = service.createReply(factory, requestHeader);
			IDCard idcard = service.createIDCard(factory);
			reply.setIDCard(idcard);
		}

		try {
			MessageContext mc = messageContext.getOperationContext().getMessageContext(messageContext.isFault() ? "Fault" : "Out");
			Axis2Util.addMessageToSOAPEnvelope(mc, reply);

			log.info("service sending: " + mc.getEnvelope());

		} catch (Throwable e) {
			log.error(e);
			throw new AxisFault(e.getMessage(), e);
		}

	}

}
