/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.axis2.module;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.RequestHeader;

/**
 * Interface to be implemented by servers using the module
 *
 * Implementations are responsible of adding SOSI ID-card to the response.
 *
 * @since 1.5.6
 */
public interface SOSIService {

    /**
     * Create the ID-card, that the module should use for authenticating web service responses.
     * @param factory
     *          Instance of <code>SOSIFactory</code> to use for ID-Card generation.
     * @return id-card
     */
    IDCard createIDCard(SOSIFactory factory);

    /**
     * Return an instance of the SOSI factory to be used for creating ID-cards authenticating the client.
     * @return
     *          instance of <code>SOSIFactory</code>.
     */
    SOSIFactory createSOSIFactory();


    /**
     * Create the SOSI reply.
     * @param factory
     *          Instance of <code>SOSIFactory</code> to use for reply generation.
     * @param requestHeader
     *          RequestHeader for which to create a reply.
     * @return SOSI reply.
     */
    Reply createReply(SOSIFactory factory, RequestHeader requestHeader);

    /**
     * Create the SOSI error reply.
     * @param factory
     *          Instance of <code>SOSIFactory</code> to use for reply generation.
     * @param requestHeader
     *          RequestHeader for which to create a reply.
     * @return SOSI reply.
     */
	Reply createErrorReply(SOSIFactory factory, RequestHeader requestHeader, String faultCode, String faultString);

    /**
     * Authorize the request for the implementing service.
     * @param idcard
     * 		IDCard in the request
     * @return
     *      true, if request is authorized, false otherwise.
     */
    boolean isAuthorized(IDCard idcard);

}
