/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.cpr;

import dk.sosi.demo.axis2.module.SOSIClient;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.CareProvider;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import junit.framework.TestCase;
import org.apache.axis2.client.Options;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.PatternLayout;

import java.rmi.RemoteException;

/**
 * @author Nymand
 */
public class CPRClientAxis2Test extends TestCase {

	private CivilRegistrationServiceServiceStub service;

	protected void setUp() throws Exception {
		super.setUp();
		PatternLayout patternLayout = new PatternLayout("%-5p %c - %m%n");
		ConsoleAppender consoleAppender = new ConsoleAppender(patternLayout);
		consoleAppender.setThreshold(Level.ERROR);
		BasicConfigurator.configure(consoleAppender);

        ConfigurationContext ctx = ConfigurationContextFactory.createConfigurationContextFromURIs(this.getClass().getResource("axis2/conf/axis2.xml"), null);

		//Axis2
		service = new CivilRegistrationServiceServiceStub(ctx, "http://localhost:8080/provider-axis2/services/CivilRegistrationService");

		//Axis1
		//service = new CivilRegistrationServiceServiceStub(ctx, "http://localhost:8080/provider/services/CivilRegistrationService");

		service._getServiceClient().engageModule("sosi");
		IDCard idcard = createIDCard(CredentialVaultUtil.createSOSIFactory());

		// Set the idcard that the axis2 module should use
		Options options = service._getServiceClient().getOptions();
		options.setProperty(SOSIClient.SOSI_IDCARD_KEY, idcard);

	}

	protected void tearDown() throws Exception {
		super.tearDown();
		service = null;
	}

    public void testNothing() {}

	public void disabledTestCPRAxis2() {
		try {
            CivilRegistrationServiceServiceStub.GetPersonDetails req = new CivilRegistrationServiceServiceStub.GetPersonDetails();
            req.setCivilRegistrationNumber("0123456789");

            CivilRegistrationServiceServiceStub.GetPersonDetailsResponse resp = service.getPersonDetails(req, null, null);
            assertEquals("John Doe", resp.getGetPersonDetailsReturn().getPersonName());
		} catch (RemoteException e) {
			System.out.println("CPRClientAxis2.testCPRAxis2() disabled: service not running");
		} catch (DgwsFaultMessage e) {
			fail(e.getFaultMessage().getFaultCode());
		}

    }

	public void disabledTestCPRAxis2ServiceException() {
        try {
            CivilRegistrationServiceServiceStub.GetPersonDetails req = new CivilRegistrationServiceServiceStub.GetPersonDetails();
            req.setCivilRegistrationNumber("TestServiceException");
            service.getPersonDetails(req, null, null);
            fail();
        } catch (DgwsFaultMessage fault) {
        	assertEquals("WST_INVALID_REQUEST", fault.getFaultMessage().getFaultCode());
        } catch (RemoteException e) {
			System.out.println("CPRClientAxis2.testCPRAxis2ServiceException() disabled: service not running");
		}
	}

    private IDCard createIDCard(SOSIFactory factory) {
        String systemName = "DemoCPR";
        String orgCVR = "12345678";
        String orgName = "Fyns Amt";
        CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, orgCVR, orgName);
        AuthenticationLevel authenticationLevel = AuthenticationLevel.VOCES_TRUSTED_SYSTEM;
        IDCard idCard = factory.createNewSystemIDCard(systemName, careProvider, authenticationLevel, null,
                null, factory.getCredentialVault().getSystemCredentialPair().getCertificate(), null);
        return idCard;
    }


}
