/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */

package dk.sosi.demo.client.pacf;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.SecurityTokenResponse;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.model.constants.NameSpaces;
import dk.sosi.seal.xml.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.Properties;

/**
 * @author sr
 * @since 1.0
 */
public class LocalTomcatServiceConnector implements ServiceConnector {

    protected String request;
    protected String response;
    protected String returnVal;
    protected String total;
    
    Properties properties = SignatureUtil.setupCryptoProviderForJVM();

    /* (non-Javadoc)
      * @see dk.sosi.demo.client.pacf.ServiceConnector#getRequest()
      */
    public String getRequest() {
        return request;
    }

    /* (non-Javadoc)
      * @see dk.sosi.demo.client.pacf.ServiceConnector#getResponse()
      */
    public String getResponse() {
        return response;
    }

    /* (non-Javadoc)
      * @see dk.sosi.demo.client.pacf.ServiceConnector#callService()
      */
    public SecurityTokenResponse callIdpService(SOSIFactory factory, Document doc) throws Exception {
        // Setup WebService
        final String SERVER = System.getProperty("sts.url", 
        	"http://pan.certifikat.dk/sts/services/SecurityTokenService");
        final String SOAP_ACTION = "";

        try {
            URL u = new URL(SERVER);
            URLConnection uc = u.openConnection();
            HttpURLConnection connection = (HttpURLConnection) uc;
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setRequestMethod("POST");
            connection.setRequestProperty("SOAPAction", SOAP_ACTION);

            OutputStream out = connection.getOutputStream();
            Writer wout = new OutputStreamWriter(out);
			String xml = XmlUtil.node2String(XmlUtil.readXml(properties, XmlUtil.node2String(doc, false, true), false), false, true);
            System.out.println("========== Sending ===========");
            System.out.println(xml);
            System.out.println("==============================");
            wout.write(xml);
            wout.flush();
            wout.close();

            InputStream in = connection.getInputStream();
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            String line;
            total = "";
            while ((line = reader.readLine()) != null) {
                total += line;
            }
            in.close();
            request = xml;
            response = total;

            System.out.println("========== Receiving ===========");
            System.out.println(response);
            System.out.println("================================");

            Document returnDoc = XmlUtil.readXml(properties, response, true);

            boolean isFault = returnDoc.getElementsByTagNameNS(NameSpaces.SOAP_SCHEMA, "Fault").getLength() > 0;
            if (!isFault) {
            	int authlvl = -1;
                NodeList attributes=  returnDoc.getElementsByTagNameNS(NameSpaces.SAML2ASSERTION_SCHEMA, "Attribute");
                for (int i = 0; i < attributes.getLength(); i++) {
                    Node elmAttr = attributes.item(i);
                    if("sosi:AuthenticationLevel".equals(elmAttr.getAttributes().item(0).getNodeValue())) {
                    	authlvl = Integer.parseInt(elmAttr.getFirstChild().getFirstChild().getNodeValue());
		                System.out.println("Authorisationlevel: "+authlvl);
		                break;
                    }
				}
                
                if(AuthenticationLevel.MOCES_TRUSTED_USER.getLevel() == authlvl || 
                		AuthenticationLevel.VOCES_TRUSTED_SYSTEM.getLevel() == authlvl) {

                	Node elmSignature = returnDoc.getElementsByTagNameNS(NameSpaces.DSIG_SCHEMA, "Signature").item(0);
                    if (!SignatureUtil.validate(elmSignature, factory.getFederation(), factory.getCredentialVault(),true))
                        System.out.println("Validation failed");
                } else {
                    System.out.println("No validation");
                }

            }

            // Build reply
            XmlUtil.readXml(properties,total, true);
            return factory.deserializeSecurityTokenResponse(total);

        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

    }

    /* (non-Javadoc)
      * @see dk.sosi.demo.client.pacf.ServiceConnector#callService()
      */
    public Reply callProviderService(SOSIFactory factory, Document doc) throws Exception {
        // Setup WebService
        final String SERVER = "http://localhost:8080/provider/services/ProviderService";
        final String SOAP_ACTION = "";

        try {
            URL u = new URL(SERVER);
            URLConnection uc = u.openConnection();
            HttpURLConnection connection = (HttpURLConnection) uc;
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setRequestMethod("POST");
            connection.setRequestProperty("SOAPAction", SOAP_ACTION);

            OutputStream out = connection.getOutputStream();
            Writer wout = new OutputStreamWriter(out);
            String xml = XmlUtil.node2String(doc, false, true);
            System.out.println("========== Sending ===========");
            System.out.println(xml);
            System.out.println("==============================");
			wout.write(xml);
            wout.flush();
            wout.close();

            InputStream in = connection.getInputStream();
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            String line;
            total = "";
            while ((line = reader.readLine()) != null) {
                total += line;
            }
            in.close();
            request = xml;
            response = total;
            System.out.println("========== Receiving ===========");
            System.out.println(response);
            System.out.println("================================");
            
        } catch (IOException e) {
            e.printStackTrace();
        }

        // Build reply
        XmlUtil.readXml(properties, total, false);
        return factory.deserializeReply(total);
    }
}
