/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.client.pacf;

import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.xml.XmlUtil;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.File;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;

public class SOSIClientFormatter implements SOSIClientReactions {
	private static final String NO_REQUEST_XML_AVAILABLE = "No Request XML avaliable";
	private static final String NO_REPLY_XML_AVAILABLE = "No Reply XML avaliable";

	private final SOSIClientPresentation presentation;

	/**
	 * Default constructor.
	 * @param presentation the presentation to act upon.
	 */
	public SOSIClientFormatter(SOSIClientPresentation presentation) {
		this.presentation = presentation;
	}

	// ==========================================
	//              ReAction methods
	// ==========================================
	public void initialize() {
		presentation.replyXML.setText(NO_REPLY_XML_AVAILABLE);
		presentation.requestXML.setText(NO_REQUEST_XML_AVAILABLE);
	}

	public void newIDCard(String xmlrequest, String xmlresponse, IDCard idCard) {
		presentation.lblID.setText(idCard.getIDCardID());
		presentation.lblCreated.setText(XmlUtil.getDateFormat(false).format(idCard.getCreatedDate()));
		presentation.lblIssuer.setText(idCard.getIssuer());
		presentation.requestXML.setText(prettyPrintXML(xmlrequest));
		presentation.replyXML.setText(prettyPrintXML(xmlresponse));
	}

    public void fault(String xmlrequest, String xmlresponse) {
        presentation.requestXML.setText(prettyPrintXML(xmlrequest));
        presentation.replyXML.setText(prettyPrintXML(xmlresponse));
    }

    public void serviceResponse(String xmlrequest, String xmlresponse, String ret) {
		presentation.lblServiceResult.setText(ret);
		presentation.requestXML.setText(prettyPrintXML(xmlrequest));
		presentation.replyXML.setText(prettyPrintXML(xmlresponse));
	}

	private String prettyPrintXML(String xml) {
		StringWriter writer = new StringWriter();
		try {
			Source xmlSource = new StreamSource(new StringReader(xml));
			Source xsltSource = new StreamSource(new File("xml2html.xsl"));
			Result result = new StreamResult(writer);
			TransformerFactory transFact = TransformerFactory.newInstance();
			Transformer trans = transFact.newTransformer(xsltSource);
			trans.transform(xmlSource, result);
		} catch (Exception e) {
			e.printStackTrace(new PrintWriter(writer));
		}
		return writer.toString();
	}
}