/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.client.pacf;

import info.clearthought.layout.TableLayout;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class SOSIClientPresentation extends JFrame {

	private static final long serialVersionUID = 1326813858084535986L;

	private static final String NO_VALUE = "No value";

	// =============================================
	// Components visible to the formatter
	// =============================================

	JEditorPane requestXML;

	JEditorPane replyXML;

	JLabel lblID;

	JLabel lblCreated;

	JLabel lblIssuer;

	JLabel lblServiceResult;

	private final SOSIClientActions controller;

	/**
	 * Default constructor. Initializes the frame.
	 */
	public SOSIClientPresentation(SOSIClientActions controller) {
		this.controller = controller;
		JTabbedPane topTabPanel = new JTabbedPane();
		JPanel idCardPanel = createIDCardPanel();
		JPanel servicePanel = createServicePanel();
		topTabPanel.addTab("ID Card Request", idCardPanel);
		topTabPanel.addTab("Ordinary Service Request", servicePanel);
		topTabPanel.setBorder(createDefaultBorder());
		topTabPanel.setPreferredSize(new Dimension(800, 200));

		requestXML = new JEditorPane();
		requestXML.setEditable(false);
		requestXML.setContentType("text/html");
		JScrollPane requestScrollPane = new JScrollPane(requestXML);
				
		replyXML = new JEditorPane();
		replyXML.setEditable(false);
		replyXML.setContentType("text/html");
		JScrollPane replyScrollPane = new JScrollPane(replyXML);

		JTabbedPane bottomTabPanel = new JTabbedPane();
		bottomTabPanel.setBorder(createDefaultBorder());
		bottomTabPanel.addTab("Request XML", requestScrollPane);
		bottomTabPanel.addTab("Reply XML", replyScrollPane);
		bottomTabPanel.setPreferredSize(new Dimension(800, 400));

		JSplitPane split = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				topTabPanel, bottomTabPanel);

		getContentPane().add(split);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setTitle("SOSI client DEMO");
		pack();
	}

	// =============================================
	// Private methods
	// =============================================
	private Border createDefaultBorder() {
		return BorderFactory.createEmptyBorder(5, 5, 5, 5);
	}

	private TableLayout getTableLayout() {
		TableLayout layout = new TableLayout(new double[][] {
				{ 100, 150, 150, 100, TableLayout.FILL },
				{ 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, TableLayout.FILL } });
		layout.setVGap(2);
		layout.setHGap(5);
		return layout;
	}

	private JPanel createIDCardPanel() {
		JPanel panel = new JPanel();
		panel.setBorder(createDefaultBorder());
		panel.setLayout(getTableLayout());

		final JComboBox cmbLevel;
		addComponent(panel, "Level:", cmbLevel = new JComboBox(new Integer[] {
				new Integer(1), new Integer(3), new Integer(4), new Integer(5) }), 0, 0);
		cmbLevel.setSelectedIndex(2);
		final JComboBox cmbRole;
		addComponent(panel, "User Role:", cmbRole = new JComboBox(new String[] {
				"doctor", "nurse" }), 1, 0);
		final JTextField txtCPR;
		addComponent(panel, "CPR:", txtCPR = new JTextField("1111111118"), 2,
				0);
		final JTextField txtGivenName;
		addComponent(panel, "Given name:",
				txtGivenName = new JTextField("Jan"), 3, 0);
		final JTextField txtSurName;
		addComponent(panel, "Sur Name:", txtSurName = new JTextField("Riis"),
				4, 0);
		final JTextField txtOccupation;
		addComponent(panel, "Occupation:", txtOccupation = new JTextField(
				"SOSI Project janitor"), 5, 0);
		final JTextField txtEMail;
		addComponent(panel, "E-mail:", txtEMail = new JTextField(
				"jan@posility.com"), 6, 0);
		final JTextField txtITSystem;
		addComponent(panel, "IT-System:", txtITSystem = new JTextField(
				"SOSITEST"), 7, 0);
		final JTextField txtOrgName;
		addComponent(panel, "Organization name:", txtOrgName = new JTextField(
				"The SOSI project"), 8, 0);
		final JTextField txtOrgCVR;
		addComponent(panel, "Organization CVR:", txtOrgCVR = new JTextField(
				"19343634"), 9, 0);

		JButton btnRequestIDCard;
		panel.add(btnRequestIDCard = new JButton("Request ID-Card >>"),
				"2,3,2,5");
		btnRequestIDCard.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					controller.requestIDCard(((Integer) cmbLevel.getSelectedItem())
							.intValue(), txtCPR.getText(), txtGivenName.getText(),
							txtSurName.getText(), txtOccupation.getText(), txtEMail
									.getText(), (String) cmbRole.getSelectedItem(),
							txtITSystem.getText(), txtOrgName.getText(), txtOrgCVR
									.getText(), true);
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
		});

		addComponent(panel, "ID-Card ID:", lblID = createDefaultLabel(), 3, 3);
		addComponent(panel, "Created:", lblCreated = createDefaultLabel(), 4, 3);
		addComponent(panel, "Issuer:", lblIssuer = createDefaultLabel(), 5, 3);

		return panel;
	}

	private JPanel createServicePanel() {
		JPanel panel = new JPanel();
		panel.setBorder(createDefaultBorder());
		panel.setLayout(getTableLayout());

		final JTextField txtArg;
		addComponent(panel, "Argument:", txtArg = new JTextField("TEST"), 2, 0);

		JButton requestService;
		panel.add(requestService = new JButton("Request Service >>"), "2,3,2,5");
		requestService.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					controller.requestService(txtArg.getText());
				} catch (Exception ee) {
					ee.printStackTrace();
				}
			}
		});

		addComponent(panel, "Result:", lblServiceResult = createDefaultLabel(),	3, 3);
		return panel;
	}

	private JLabel createDefaultLabel() {
		JLabel label = new JLabel(NO_VALUE);
		label.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		return label;
	}

	private void addComponent(JPanel panel, String label, JComponent component,
			int row, int col) {
		panel.add(new JLabel(label), col + "," + row + ",r,c");
		panel.add(component, (col + 1) + "," + row + ",f,c");
	}
}
