/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.cpr;

import dk.sosi.demo.client.vault.CredentialVaultUtil;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.CareProvider;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;

/**
 * Implementation of SOSIClientResponseHandler.
 *
 * @author Simon
 * @since 1.3
 */
public class CPRClientRequestHandler extends SOSIClientRequestHandler {

	protected SOSIFactory createSOSIFactory() {
		return CredentialVaultUtil.createSOSIFactory(null);
	}

	protected IDCard createIDCard(SOSIFactory factory) {
		String systemName = "DemoCPR";
        String orgCVR = "12345678";
        String orgName = "Fyns Amt";
        CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, orgCVR, orgName);
        AuthenticationLevel authenticationLevel = AuthenticationLevel.VOCES_TRUSTED_SYSTEM;
		IDCard idCard = factory.createNewSystemIDCard(systemName, careProvider, authenticationLevel, null,
				null, factory.getCredentialVault().getSystemCredentialPair().getCertificate(), null);
		return idCard;
	}

	protected Request createRequest(SOSIFactory factory) {
		String flowID = null; // only necessary when multiple calls to service (session)
		return factory.createNewRequest(false, flowID);
	}

}
