/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.cpr;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.AxisUtil;
import org.apache.axis.AxisFault;
import org.apache.axis.MessageContext;
import org.apache.axis.handlers.BasicHandler;
import org.apache.axis.message.SOAPEnvelope;

/**
 * Generic SOSIClientRequestHandler.
 * 
 * @author Simon
 * @since 1.3
 */
public abstract class SOSIClientRequestHandler extends BasicHandler {

	public SOSIFactory factory;
	public Request request;
	public Reply reply;

	//TODO make abstract and callback to handle userspecific info
	public void invoke(MessageContext msgContext) throws AxisFault {
		// Get the factory
        SOSIFactory factory = createSOSIFactory();
		msgContext.setProperty("sosi:factory", factory);

        // Create IDCard
        IDCard idCard = createIDCard(factory);

		// Create Request
		Request request = createRequest(factory);
		request.setIDCard(idCard);

		try {
			// Make sure that all namespaces are present in the axis response
			// This is an axis specific requirement
			SOAPEnvelope req = msgContext.getRequestMessage().getSOAPEnvelope();
			AxisUtil.addMessageToSOAPEnvelope(req, request);

		} catch (Throwable e) {
			throw new AxisFault(e.getMessage(), e);
		}
	}

	protected abstract SOSIFactory createSOSIFactory();

	protected abstract IDCard createIDCard(SOSIFactory factory);

	protected abstract Request createRequest(SOSIFactory factory);
}