/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.client.pacf;

import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.IDCard;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Unit test for simple App.
 */
public class SOSIClientTest extends TestCase {

	/**
	 * Create the test case
	 *
	 * @param testName
	 *            name of the test case
	 */
	public SOSIClientTest(String testName) {
		super(testName);
	}

	/**
	 * @return the suite of tests being tested
	 */
	public static Test suite() {
		return new TestSuite(SOSIClientTest.class);
	}

	// =================================
	// Test cases
	// =================================

	public void testInitializeConnection() throws Exception  {
		TestReactor testReactor = new TestReactor();
		createController(testReactor); // Implicit initialization
		checkReactorInitialization(testReactor);
	}

	public void testNewIDCardConnection() throws Exception  {
		final BooleanHolder resultHolder = new BooleanHolder();
		TestReactor testReactor = new TestReactor() {
			public void newIDCard(String xmlrequest, String xmlresponse, IDCard arg0) {
				resultHolder.setBoolean(true);
			}
		};
		SOSIClientController controller = createController(testReactor);
		checkReactorInitialization(testReactor);

		try {
			controller.requestIDCard(AuthenticationLevel.MOCES_TRUSTED_USER.getLevel(),"1234567890", "givenName", "surName", "occupation", "email", "role", "systemName", "orgName", "orgCVR", true);
		} catch (Exception e) {
			e.printStackTrace();
			assertTrue("Exception: " + e, false);
		}
		assertTrue("The expected method was not called", resultHolder.getResult());
	}

	public void testSPConnection() throws Exception  {
/*		final BooleanHolder resultHolder = new BooleanHolder();
		TestReactor testReactor = new TestReactor() {
			public void serviceResponse(String req, String resp, String ret) {
				resultHolder.setBoolean(true);
			}
		};
		SOSIClientController controller = createController(testReactor);
		checkReactorInitialization(testReactor);

		try {
			controller.requestService("test");
		} catch (Exception e) {
			assertTrue("Exception: " + e, false);
		}
		assertTrue("The expected method was not called", resultHolder.getResult());
		*/
	}

	// =================================
	// Private stuff
	// =================================

	private void checkReactorInitialization(TestReactor testReactor) {
		assertTrue("The reactor realization was not initialized", testReactor.initialized);
	}

	private SOSIClientController createController(SOSIClientReactions reactor) {
		SOSIClientController controller = new SOSIClientController();
		controller.initialize(new MockServiceConnector(), reactor);
		return controller;
	}

	private class TestReactor implements SOSIClientReactions {
		boolean initialized = false;
		public void initialize() {
			initialized  = true;
		}
		public void newIDCard(String xmlrequest, String xmlresponse, IDCard idCard) {
			fail("This method should not get called!");
		}
		public void serviceResponse(String req, String resp, String ret) {
			fail("This method should not get called!");
		}
        public void fault(String request, String response) {
            fail("This method should not get called!");
        }
    }
}
