/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.cpr;

import junit.framework.TestCase;
import junit.textui.TestRunner;
import org.apache.axis.EngineConfiguration;

import javax.xml.rpc.ServiceException;
import java.rmi.RemoteException;

/**
 * @since 1.3
 */
public class CPRClientTest extends TestCase {
	CivilRegistrationService port;

	protected void setUp() throws Exception {
		// Make a service
		EngineConfiguration clientConfig=CPRClientUtils.createClientConfig(); 
		CivilRegistrationServiceService service = new CivilRegistrationServiceServiceLocator(clientConfig);

		// Now use the service to get a stub which implements the SDI.
		port = service.getCivilRegistrationService();
	}


	public void testCPRAxis() throws ServiceException, RemoteException {
		try {
			PersonDetails pd = port.getPersonDetails("0123456789");
			assertEquals("John Doe",pd.getPersonName());
		} catch (RemoteException e) {
			if (e.getMessage().indexOf("Connection refused") != -1 || e.getMessage().indexOf("404") != -1) {
				System.out.println("CPRClientTest.testCPRAxis() disabled: service not running");
			} else {
                throw e;
			}
		}
	}

	public void testCPRAxisServiceException() throws RemoteException {
		try {
			port.getPersonDetails("TestServiceException");
			fail();
		} catch (DgwsFaultMessage e) {
			assertEquals("WST_INVALID_REQUEST", e.getFault_medcom());
		} catch (RemoteException e) {
			if (e.getMessage().indexOf("Connection refused") != -1 || e.getMessage().indexOf("404") != -1) {
				System.out.println("CPRClientTest.testCPRAxisServiceException() disabled: service not running");
			} else {
                throw e;
			}
		}
	}

	public void testCPRAxisServiceHandlerException() throws RemoteException {
		try {
			port.getPersonDetails("TestServiceHandlerException");
			fail();
		} catch (DgwsFaultMessage e) {
			assertEquals("WST_INVALID_REQUEST", e.getFault_medcom());
		} catch (RemoteException e) {
			if (e.getMessage().indexOf("Connection refused") != -1 || e.getMessage().indexOf("404") != -1) {
				System.out.println("CPRClientTest.testCPRAxisServiceHandlerException() disabled: service not running");
			} else {
                throw e;
			}
		}
	}

	public static void main(String[] args) {
		TestRunner.run(CPRClientTest.class);
	}

}

