/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.cpr;

import dk.sosi.demo.axis2.module.SOSIService;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.*;
import dk.sosi.seal.model.constants.FlowStatusValues;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import dk.sosi.seal.pki.DistinguishedName;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @since 1.5.6
 */
public class CPRSOSIService implements SOSIService {
    private final Log log = LogFactory.getLog(this.getClass());

    public IDCard createIDCard(SOSIFactory factory) {
        String systemName = "DemoCPRProvider";
        String orgCVR = "12345678";
        String orgName = "Fyns Amt";
        CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, orgCVR, orgName);

        AuthenticationLevel authenticationLevel = AuthenticationLevel.VOCES_TRUSTED_SYSTEM;
        IDCard idCard = factory.createNewSystemIDCard(systemName, careProvider, authenticationLevel, null,
                null, factory.getCredentialVault().getSystemCredentialPair().getCertificate(), null);
        return idCard;
    }

    public SOSIFactory createSOSIFactory() {
        return CredentialVaultUtil.createSOSIFactory();
    }

    public Reply createReply(SOSIFactory factory, RequestHeader requestHeader) {
    	return factory.createNewReply(requestHeader.getDGWSVersion(), requestHeader.getMessageID(), requestHeader.getFlowID(),
    			FlowStatusValues.FLOW_FINALIZED_SUCCESFULLY);
    }

    public Reply createErrorReply(SOSIFactory factory, RequestHeader requestHeader, String faultCode, String faultString) {
    	return factory.createNewErrorReply(requestHeader.getDGWSVersion(), requestHeader.getMessageID(), requestHeader.getFlowID(),
    			faultCode, faultString);
    }

    public boolean isAuthorized(IDCard idCard) {
        log.info("Service authorizing card signed by: " + new DistinguishedName(idCard.getSignedByCertificate().getSubjectX500Principal()));
        return true;
    }

}
