/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.cpr;

import dk.sosi.seal.model.Reply;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;

import javax.servlet.*;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Servlet filter used to replace Axis internal faults with invalid request
 * fault as specified by DGWS. See also "SOSI STS Teknisk beskrivelse" v1.0
 * 
 * Thanks to the STS Project Team.
 *
 * @since 1.5
 */
public abstract class SOSIAxisExceptionHandlerFilter implements Filter {
	private static Log log = LogFactory.getLog(SOSIAxisExceptionHandlerFilter.class);

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) {
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
	}

	public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
		CharResponseWrapper wrappedRes = new CharResponseWrapper((HttpServletResponse)res);
		try {
			log.debug("doFilter");
			chain.doFilter(req, wrappedRes);
		} finally {
			try {
				ServletOutputStream out = res.getOutputStream();
				
				// catch all errors 5xx
				if ((wrappedRes.getStatusCode() / 100) == 5) {
					log.debug("replace axis response");
					String respxml = new String(wrappedRes.getBytes());
					
					Reply reply = handleError(respxml);
					
					Document doc = reply.serialize2DOMDocument();
					String xml = XmlUtil.node2String(doc);
					out.write(xml.getBytes());
				} else {
					out.write(wrappedRes.getBytes());
				}
			} catch (Throwable e) {
				throw new ServletException(e);
			}
		}
	}

	protected abstract Reply handleError(String xml);

	private static class CharResponseWrapper extends HttpServletResponseWrapper {
		private ByteArrayPrintWriter bapw = new ByteArrayPrintWriter();
		private int status;

		public CharResponseWrapper(HttpServletResponse response) {
			super(response);
		}
		
		public int getStatusCode() {
			return status;
		}
		
		public byte[] getBytes() {
			return bapw.toByteArray();
		}
		

		public PrintWriter getWriter() {
			return bapw.getWriter();
		}

		public ServletOutputStream getOutputStream() {
			return bapw.getStream();
		}

		public void sendError(int sc, String msg) throws IOException {
			this.status = sc;
			super.sendError(sc, msg);
		}

		public void sendError(int sc) throws IOException {
			this.status = sc;
			super.sendError(sc);
		}

		public void setStatus(int sc, String msg) {
			this.status = sc;
			super.setStatus(sc, msg);
		}

		public void setStatus(int sc) {
			this.status = sc;
			super.setStatus(sc);
		}

		public String toString() {
			return bapw.toString();
		}
	}
	
	  private static class ByteArrayPrintWriter {
		private ByteArrayOutputStream baos = new ByteArrayOutputStream();
		private PrintWriter pw = new PrintWriter(baos);
		private ServletOutputStream sos = new ByteArrayServletStream(baos);

		public PrintWriter getWriter() {
			return pw;
		}

		public ServletOutputStream getStream() {
			return sos;
		}

		byte[] toByteArray() {
			return baos.toByteArray();
		}
	}

	private static class ByteArrayServletStream extends ServletOutputStream {
		ByteArrayOutputStream baos;

		ByteArrayServletStream(ByteArrayOutputStream baos) {
			this.baos = baos;
		}

		public void write(int param) {
			baos.write(param);
		}
	}
}
