/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.cpr;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.model.constants.FlowStatusValues;
import dk.sosi.seal.modelbuilders.SignatureInvalidModelBuildException;
import org.apache.axis.AxisFault;
import org.apache.axis.MessageContext;
import org.apache.axis.handlers.BasicHandler;

/**
* @since 1.5
*/
public abstract class SOSIRequestHandler extends BasicHandler {

	public SOSIFactory factory;
	public Request request;
	public Reply reply;

	public void invoke(MessageContext msgContext) throws AxisFault {
		try {
			String xml = msgContext.getCurrentMessage().getSOAPPartAsString();
			//FIXME should be possible to avoid this regex via the right axis props setup
			// remove indention added by axis
			xml = removeFormatting(xml);

			factory = createFactory();

			// rebuild request from actual SOAP message rather than from req.toString(). Axis specific behaviour.
			//TODO validate request
			request = factory.deserializeRequest(xml);

			reply = factory.createNewReply(request, FlowStatusValues.FLOW_FINALIZED_SUCCESFULLY);

			// add idcard from request
			//TODO return provider idcard
			reply.setIDCard(request.getIDCard());


			// Test exception thrown from handler
			if(xml.indexOf("TestServiceHandlerException") >= 0) throw new AxisFault("TestServiceHandlerException");

			msgContext.setProperty("sosi:factory", factory);
			msgContext.setProperty("sosi:request", request);
			msgContext.setProperty("sosi:reply", reply);
		} catch (SignatureInvalidModelBuildException e) {
			throw new AxisFault("TestServiceHandlerException", e);
		} catch (Throwable t) {
			throw new AxisFault("Error in SOSIRequestHandler", t);
		}
	}

	protected abstract SOSIFactory createFactory();

	private String removeFormatting(String xml) {
		return xml.replaceAll(">\\s*<", "><");
	}
}