/*
 * The SOSI OSS Library - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of the SOSI OSS Library
 * is copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 */
package dk.sosi.demo.provider;

import dk.sosi.demo.provider.vault.CredentialVaultUtil;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.model.constants.DGWSConstants;
import dk.sosi.seal.model.constants.FaultCodeValues;
import dk.sosi.seal.model.constants.FlowStatusValues;
import dk.sosi.seal.modelbuilders.SignatureInvalidModelBuildException;
import dk.sosi.seal.pki.SOSITestFederation;
import dk.sosi.seal.xml.AxisUtil;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.axis.MessageContext;
import org.apache.axis.message.SOAPEnvelope;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.Date;
import java.util.Properties;

/**
 * Simple Axis webservice provider
 *
 * @author Søren Raaschou
 * @since 1.0
 */
public class ProviderApp {

    public void serverTime(SOAPEnvelope req, SOAPEnvelope resp) {

        // Get message context
        MessageContext msgContext = MessageContext.getCurrentContext();

        //try {
        Reply sosiReply;
        Properties properties = SignatureUtil.setupCryptoProviderForJVM();
        SOSIFactory factory = CredentialVaultUtil.createSOSIFactory(new SOSITestFederation(properties));
        try {

				// Build sosiRequest object implies validation of Signature
			    Request sosiRequest = factory.deserializeRequest(msgContext.getCurrentMessage().getSOAPPartAsString());
			    sosiReply = factory.createNewReply(sosiRequest, FlowStatusValues.FLOW_FINALIZED_SUCCESFULLY);

				Document doc1 = XmlUtil.createEmptyDocument();
			    Date date = new Date();
			    String strDate = date.toString();
			    Element body = doc1.createElement("ServerTime");
			    body.appendChild(doc1.createTextNode(strDate));
			    sosiReply.setBody(body);
				AxisUtil.addBodyToSOAPEnvelope(resp, sosiReply.serialize2DOMDocument());
			} catch (SignatureInvalidModelBuildException e) {
			    sosiReply = factory.createNewErrorReply(DGWSConstants.VERSION_1_0_1, e.getMessageID(), e.getFlowID(), FaultCodeValues.INVALID_SIGNATURE, "The supplied signature was invalid");
			} catch (Exception e) {
			    throw new RuntimeException("Fatal: Unable to get SOAP document from Seal", e);
			}
			//AxisUtil.addMessageToSOAPEnvelope(resp, sosiReply);
//		} catch (SOAPException e) {
//            throw new RuntimeException("Fatal: Unable to get SOAP document from Seal", e);
//		}
    }
}
