/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.seal.common.functionalExt.either;

import java.util.function.Function;

public abstract class Either<L, R> {
    public abstract  <RR> RR match(Function<L, RR> left, Function<R, RR> right);

    private static class Left<L, R> extends Either<L, R> {
        private final L value;
        private Left(L value) { this.value = value; }

        @Override
        public String toString() { return String.format("Left(%s)", value); }

        @Override
        public <RR> RR match(Function<L, RR> left, Function<R, RR> right) { return left.apply(value); }
    }

    private static class Right<L, R> extends Either<L, R> {
        private final R value;
        private Right(R value) { this.value = value; }
        @Override
        public String toString() { return String.format("Right(%s)", value); }

        @Override
        public <RR> RR match(Function<L, RR> left, Function<R, RR> right) { return right.apply(value); }
    }

    public static <T, U> Either<T, U> left(T value) { return new Left<>(value); }
    public static <T, U> Either<T, U> right(U value) { return new Right<>(value); }
}
