/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.keyresolver.internal;

import java.security.cert.X509Certificate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.WSDocInfo;
import org.apache.ws.security.WSDocInfoStore;
import org.apache.ws.security.WSSecurityException;
import org.apache.ws.security.message.token.BinarySecurity;
import org.apache.xml.security.keys.keyresolver.KeyResolverException;
import org.apache.xml.security.keys.keyresolver.implementations.X509CertificateResolver;
import org.apache.xml.security.keys.storage.StorageResolver;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import dk.sosi.seal.model.CitizenIdentityToken;
import dk.sosi.seal.transform.internal.SecurityTokenReference;
import dk.sosi.seal.xml.CertificateParser;

public class STRResolver extends X509CertificateResolver {
	/** {@link org.apache.commons.logging} logging facility */
	static Log log = LogFactory.getLog(STRResolver.class.getName());

	/**
	 * Method engineResolveX509Certificate
	 * 
	 * @inheritDoc
	 * @param element
	 * @param BaseURI
	 * @param storage
	 *
	 * @throws KeyResolverException
	 */
	public X509Certificate engineLookupResolveX509Certificate(Element element, String BaseURI, StorageResolver storage) throws KeyResolverException {

		if (!SecurityTokenReference.SECURITY_TOKEN_REFERENCE.equals(element.getLocalName())) {
			return null;
		}

		X509Certificate cert = null;

		try {
			SecurityTokenReference str = new SecurityTokenReference(element);

			Element bstElm = null;
			final Document ownerDocument = element.getOwnerDocument();
			WSDocInfo docInfo = WSDocInfoStore.lookup(ownerDocument);

			/* Tjekker både for reference og keyidentifer. */
			if (cert == null && str.containsReference()) {
				bstElm = str.getTokenElement(ownerDocument, docInfo, null);
				if (bstElm == null) {
					return null;
				}

				BinarySecurity bst = new BinarySecurity(bstElm);

				cert = CertificateParser.asCertificate(bst.getToken());
			}

			if (cert == null && str.containsKeyIdentifier()) {
				bstElm = str.getKeyIdentifierTokenElement(ownerDocument, docInfo, null);
				CitizenIdentityToken identityToken = new CitizenIdentityToken(bstElm);
				cert = CertificateParser.asCertificate(identityToken.getToken());
			}

		} catch (WSSecurityException ex) {
			switch (ex.getErrorCode()) {
				case WSSecurityException.INVALID_SECURITY_TOKEN : {
					log.info("BinarySecurity blev oprettet med forkerte tokens.", ex);
					break;
				}
				default : {
					log.debug("XMLSecurityException", ex);
					break;
				}
			}
		}
		return cert;
	}
}
