/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model;

import java.util.Arrays;
import java.util.Optional;

public interface AssuranceLevel {
	enum Type {
		NIST, NSIS;
	}
	
	enum NIST implements AssuranceLevel {
		NIVEAU_1("1"), NIVEAU_2("2"), NIVEAU_3("3"), NIVEAU_4("4"), NIVEAU_5("5");

		private final String assuranceLevel;

		NIST(String assuranceLevel) {
			this.assuranceLevel = assuranceLevel;
		}

		@Override
		public String toString() {
			return assuranceLevel;
		}
		
		@Override
		public Type getType() {
			return Type.NIST;
		}

	}
	
	/**
	 * https://digst.dk/media/22920/nsis-engelsk-version-201_final.pdf
	 */
	enum NSIS implements AssuranceLevel{
		/*LAV, BETYDELIG, HØJ*/
		Low, Substantial, High;

		@Override
		public Type getType() {
			return Type.NSIS;
		}
	}
	
	Type getType();

	static AssuranceLevel ofValue(String value) {
		Optional<NIST> nistValue = Arrays.stream(NIST.values()).filter(nist -> nist.toString().equals(value)).findAny();
		if (nistValue.isPresent()) {
			return nistValue.get();
		}
		Optional<NSIS> nsisValue = Arrays.stream(NSIS.values()).filter(nsis -> nsis.toString().equals(value)).findAny();
		if (nsisValue.isPresent()) {
			return nsisValue.get();
		}
		throw new IllegalArgumentException("No AssuranceLevel with value " + value);
	}
}
