package dk.sosi.seal.model;

import dk.sosi.seal.model.constants.NameSpaces;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import dk.sosi.seal.model.constants.BlurringInstructions10Tags;

import java.util.ArrayList;
import java.util.List;

import static dk.sosi.seal.model.BlurredOrganisation.*;
import static dk.sosi.seal.model.BlurredOrganisation.OrgType.*;
import static dk.sosi.seal.model.BlurredOrganisation.Reason.*;

public class BlurringInstructions10DomBuilder extends BlurringInstructionsDomBuilder<Document> {

    public BlurringInstructions10DomBuilder(String salt) { this.salt = salt; }

    protected BlurringInstructions10DomBuilder(BlurringInstructions blurringInstructions) {
        BlurringInstructions10Impl blurringInstructions10Impl = (BlurringInstructions10Impl)blurringInstructions;
        salt = blurringInstructions10Impl.getSalt();
        blurredOrganisations = blurringInstructions10Impl.getBlurredOrganisations();
    }

    private List<BlurredOrganisation> blurredOrganisations = new ArrayList<>();
    private final String salt;

    public BlurringInstructions10DomBuilder addBlurringInstruction(String cvr, OrgType orgType, Reason reason) {
        return addBlurringInstruction(new BlurredOrganisation(cvr, orgType, reason));
    }

    public BlurringInstructions10DomBuilder addBlurringInstruction(BlurredOrganisation o) {
        blurredOrganisations.add(o);
        return this;
    }

    @Override
    public Document build() throws ModelException {
        return createDocument();
    }

    @Override
    protected Element createRoot(Document doc) {
        Element rootElm = createElement(BlurringInstructions10Tags.BLURRING_INSTRUCTIONS);
        rootElm.setAttributeNS(null, BlurringInstructions10Tags.SALT_ATT, this.salt);
        return rootElm;
    }

    @Override
    protected void addRootAttributes(Element root) {
        addNS(root, NameSpaces.NS_BIP, NameSpaces.BLURRING_INSTRUCTIONS_PROFILE_1_0);
    }

    @Override
    protected void appendToRoot(Document localDoc, Element orgListElm) {
        for (BlurredOrganisation org : getBlurredOrganisations()) {
            Element groupElm = createElement(BlurringInstructions10Tags.BLUR_EMPLOYEE_NAMES_FROM_ORG);
            groupElm.setAttributeNS(null, BlurringInstructions10Tags.ORGTYPE_ATT, toSamlAttribute(org.getOrgType()));
            groupElm.setAttributeNS(null, BlurringInstructions10Tags.REASON_ATT, toSamlAttribute(org.getReason()));
            groupElm.setTextContent(org.getValue());
            orgListElm.appendChild(groupElm);
        }
    }

    private String toSamlAttribute(Reason r) {
        String s = r == SpecificForPerson ? "specific_for_person" :
                   r == FromRelatedPerson ? "from_related_person" :
                   null;
        if(s == null) { throw new RuntimeException(r + " could not be mapped to SAML attribute"); }
        return s;
    }

    private String toSamlAttribute(OrgType o) {
        String s = o == Cvr ? "CVR" :
                   o == Sor ? "SOR" :
                   null;
        if(s == null) { throw new RuntimeException(o + " could not be mapped to SAML attribute"); }
        return s;
    }

    @Override
    protected void validateBeforeBuild() throws ModelException { }

    public List<BlurredOrganisation> getBlurredOrganisations() { return blurredOrganisations; }
}
