package dk.sosi.seal.model;

import dk.sosi.seal.model.constants.BlurringInstructions11Tags;
import org.w3c.dom.Element;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static dk.sosi.seal.model.BlurredOrganisation.*;
import static dk.sosi.seal.model.BlurredOrganisation.OrgType.*;
import static dk.sosi.seal.model.BlurredOrganisation.Reason.*;
import static dk.sosi.seal.xml.XmlUtil.getChildElements;
import static java.util.Optional.*;

public class BlurringInstructions11Impl extends AbstractBlurringInstructions implements BlurringInstructions11 {

    private final List<BlurredOrganisation> blurredOrganisations = new ArrayList<>();
    private String salt;

    public BlurringInstructions11Impl(Element root) {
        setup(root);
    }

    @Override
    public List<BlurredOrganisation> getBlurredOrganisations() { return blurredOrganisations; }

    @Override
    public String getSalt() { return salt; }

    @Override
    protected String getXsdPath() {
        return "/blurring-instructions-1.1.xsd";
    }

    private void setup(Element root) {
        validate(root);
        parse(root);
    }

    protected void parse(Element root) {
        salt =  root.getAttributeNS(null, BlurringInstructions11Tags.SALT_ATT);

        List<Element> allBlurredOrganisations = getChildElements(root, BlurringInstructions11Tags.BLUR_EMPLOYEE_NAMES_FROM_ORG.getTagName());
        for (Element org : allBlurredOrganisations) {
            blurredOrganisations.add(
                    new BlurredOrganisation(org.getTextContent(),
                            fromOrgTypeSamlAttribute(org.getAttributeNS(null, BlurringInstructions11Tags.ORGTYPE_ATT)),
                            fromReasonSamlAttribute(org.getAttributeNS(null, BlurringInstructions11Tags.REASON_ATT))));
        }
    }

    private Reason fromReasonSamlAttribute(String s) {
        Optional<Reason> r =
                s.equals("specific_for_person") ? of(SpecificForPerson) :
                s.equals("from_related_person") ? of(FromRelatedPerson) :
                s.equals("specific_department") ? of(SpecificDepartment) :
                        empty();
        return r.orElseThrow(() -> new RuntimeException(s + " could not be mapped to " + Reason.class));
    }

    private OrgType fromOrgTypeSamlAttribute(String s) {
        Optional<OrgType> o =
                s.equals("CVR") ? of(Cvr) :
                s.equals("SOR") ? of(Sor) :
                s.equals("SHAK") ? of(Shak) :
                        empty();
        return o.orElseThrow(() -> new RuntimeException(s + " could not be mapped to " + OrgType.class));
    }
}
