/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.model;

import dk.sosi.seal.model.constants.OIOSAMLAttributes;
import dk.sosi.seal.model.constants.WSSETags;
import dk.sosi.seal.model.constants.WST14Tags;
import dk.sosi.seal.xml.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import java.util.Optional;

import static dk.sosi.seal.model.constants.HealthcareSAMLAttributes.*;
import static java.util.Optional.ofNullable;

/**
 */
public class JWTToIdentityTokenRequest extends OIOWSTrustRequest {

    public JWTToIdentityTokenRequest(Document doc) {
        super(doc);
    }

    public String getJsonWebToken() {
        Element actAsElement = getActAsElement();
        final Element bst = getTag(actAsElement, WST14Tags.actAs, WSSETags.binarySecurityToken);
        return XmlUtil.getTextNodeValue(bst);
    }

    public String getCPRNumberClaim() {
        Element claim = getClaimByUri(OIOSAMLAttributes.CPR_NUMBER);
        return claim != null ? claim.getTextContent() : null;
    }

    public String getProcurationForCprClaim() {
        return getOnBehalfOfClaim(THROUGH_PROCURATION_BY_CPR + ':').orElse(null);
    }

    public String getWardCustodyClaim() {
        return getOnBehalfOfClaim(THROUGH_WARD_CUSTODY_BY_CPR + ':').orElse(null);
    }

    public String getPartlyWardCustodyClaim() {
        return getOnBehalfOfClaim(THROUGH_PARTLY_WARD_CUSTODY_BY_CPR + ':').orElse(null);
    }

    public String getParentalCustodyClaim() {
        return getOnBehalfOfClaim(THROUGH_PARENTAL_CUSTODY_BY_CPR + ':').orElse(null);
    }

    private Optional<String> getOnBehalfOfClaim(String prefix) {
        return ofNullable(getClaimByUri(ON_BEHALF_OF))
                .map(Node::getTextContent)
                .filter(claim -> claim.startsWith(prefix))
                .map(claim -> claim.substring(prefix.length()));
    }
}