/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.model;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import dk.sosi.seal.model.constants.HealthcareSAMLAttributes;
import dk.sosi.seal.model.constants.OIOSAMLAttributes;
import dk.sosi.seal.model.constants.WSSETags;
import dk.sosi.seal.model.constants.WST14Tags;
import dk.sosi.seal.xml.XmlUtil;

/**
 */
public class JWTToOIOSAMLAssertionRequest extends OIOWSTrustRequest {

    public JWTToOIOSAMLAssertionRequest(Document doc) {
        super(doc);
    }
    
    public String getJsonWebToken() {
        Element actAsElement = getActAsElement();
        final Element bst = getTag(actAsElement, WST14Tags.actAs, WSSETags.binarySecurityToken);
        return XmlUtil.getTextNodeValue(bst);
    }

    public String getCPRNumberClaim() {
        Element claim = getClaimByUri(OIOSAMLAttributes.CPR_NUMBER);
        return claim != null ? claim.getTextContent() : null;
    }

    public String getProcurationForCprClaim() {
        Element claim = getClaimByUri(HealthcareSAMLAttributes.ON_BEHALF_OF);
        if (claim != null) {
            final String textContent = claim.getTextContent();
            final String prefix = HealthcareSAMLAttributes.THROUGH_PROCURATION_BY_CPR + ':';
            if (textContent.startsWith(prefix)) {
                return textContent.substring(prefix.length());
            }
        }
        return null;
    }
}