/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.model;

import dk.sosi.seal.model.OIOBSTSAMLAssertion.SAMLAttribute;
import dk.sosi.seal.model.constants.SAMLValues;
import org.w3c.dom.Element;

/**
 * Builder for OIO3BST assertions used for citizens
 */
public class OIO3BSTCitizenSAMLAssertionBuilder extends AbstractOIOBSTSAMLAssertionBuilder<OIO3BSTCitizenSAMLAssertionBuilder, OIO3BSTCitizenSAMLAssertion> {
	public OIO3BSTCitizenSAMLAssertionBuilder() {
		addAttribute(OIO3BSTCitizenSAMLAssertion.Attribute.SPEC_VER, "OIO-SAML-3.0");
	}

	@Override
	protected String getNameIDFormat() {
		return SAMLValues.NAMEID_FORMAT_PERSISTENT;
	}

	@Override
	protected SAMLAttribute[] getAttributes() {
		return OIO3BSTCitizenSAMLAssertion.Attribute.values();
	}

	protected Element createAttributeElement(OIO3BSTCitizenSAMLAssertion.Attribute attribute, String value) {
		return super.createAttributeElement(attribute.getName(), attribute.getNameFormat(), null, value);
	}

	public OIO3BSTCitizenSAMLAssertionBuilder setAssuranceLevel(AssuranceLevel.NSIS nsis) {
		return addAttribute(OIO3BSTCitizenSAMLAssertion.Attribute.ASSURANCE_LEVEL, nsis);
	}

	public OIO3BSTCitizenSAMLAssertionBuilder setCpr(String cpr) {
		return addAttribute(OIO3BSTCitizenSAMLAssertion.Attribute.CPR, cpr);
	}

	protected OIO3BSTCitizenSAMLAssertion createAssertion(Element elm) {
		return new OIO3BSTCitizenSAMLAssertion(elm);
	}
}
