/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.model;

import org.w3c.dom.Element;

import dk.sosi.seal.model.OIOBSTSAMLAssertion.SAMLAttribute;
import dk.sosi.seal.model.constants.SAMLValues;

/**
 */
public class OIO3BSTSAMLAssertionBuilder extends AbstractOIOBSTSAMLAssertionBuilder<OIO3BSTSAMLAssertionBuilder, OIO3BSTSAMLAssertion> {
	public OIO3BSTSAMLAssertionBuilder() {
		addAttribute(OIO3BSTSAMLAssertion.Attribute.SPEC_VER, "OIO-SAML-3.0");
	}

	@Override
	protected String getNameIDFormat() {
		return SAMLValues.NAMEID_FORMAT_PERSISTENT;
	}

	@Override
	protected SAMLAttribute[] getAttributes() {
		return OIO3BSTSAMLAssertion.Attribute.values();
	}

	protected Element createAttributeElement(OIO3BSTSAMLAssertion.Attribute attribute, String value) {
		return super.createAttributeElement(attribute.getName(), attribute.getNameFormat(), null, value);
	}

	public OIO3BSTSAMLAssertionBuilder setAssuranceLevel(AssuranceLevel any) {
		switch (any.getType()) {
			case NIST :
				return removeAttribute(OIO3BSTSAMLAssertion.Attribute.ASSURANCE_LEVEL_NSIS).addAttribute(OIO3BSTSAMLAssertion.Attribute.ASSURANCE_LEVEL_NIST, any);
			case NSIS :
				return removeAttribute(OIO3BSTSAMLAssertion.Attribute.ASSURANCE_LEVEL_NIST).addAttribute(OIO3BSTSAMLAssertion.Attribute.ASSURANCE_LEVEL_NSIS, any);
			default :
				throw new ModelException("Unkown AssuranceLevel:" + any.getType());
		}
	}

	public OIO3BSTSAMLAssertionBuilder setUuid(String uuid) {
		return addAttribute(OIO3BSTSAMLAssertion.Attribute.UUID_PROFESSIONAL, uuid);
	}

	public OIO3BSTSAMLAssertionBuilder setCpr(String cpr) {
		return addAttribute(OIO3BSTSAMLAssertion.Attribute.CPR, cpr);
	}

	public OIO3BSTSAMLAssertionBuilder setCvr(String cvr) {
		return addAttribute(OIO3BSTSAMLAssertion.Attribute.CVR, cvr);
	}

	public OIO3BSTSAMLAssertionBuilder setOrganizationName(String name) {
		return addAttribute(OIO3BSTSAMLAssertion.Attribute.NAME_ORGANIZATION, name);
	}

	protected OIO3BSTSAMLAssertion createAssertion(Element elm) {
		return new OIO3BSTSAMLAssertion(elm);
	}
}
