package dk.sosi.seal.model;

import org.w3c.dom.Element;

import dk.sosi.seal.model.constants.OIOSAML3Attributes;
import dk.sosi.seal.model.constants.OIOSAMLAttributes;

/**
 * Factory class used for creating the correct OIOBSTSAMLAssertion object, based on an assertion element.
 */
public class OIOBSTSAMLAssertionFactory {
	/**
	 * 
	 * @param assertionElm Assertion element, used for determining the OIOBSTSAMLAssertion object.
	 * @return OIOBSTSAMLAssertion based on input element.
	 * @throws ModelException If a OIOBSTSAMLAssertion can't be determined from the input
	 * @see OIOH2BSTSAMLAssertion
	 * @see OIOH3BSTSAMLAssertion
	 * @see OIO3BSTSAMLAssertion
	 */
	public static OIOBSTSAMLAssertion createOIOBSTSAMLAssertion(Element assertionElm) {
    	String specVersion = OIOBSTSAMLAssertion.getAttributeValue(assertionElm, OIOSAML3Attributes.SPEC_VERSION_ADDITIONAL, OIOSAML3Attributes.SPEC_VERSION, OIOSAMLAttributes.SPEC_VERSION);
		switch (specVersion == null ? "" : specVersion) {
			case "OIO-SAML-H-3.0": return new OIOH3BSTSAMLAssertion(assertionElm);
			case "OIO-SAML-3.0": return new OIO3BSTSAMLAssertion(assertionElm);
			case "DK-SAML-2.0": return new OIOH2BSTSAMLAssertion(assertionElm);
			default: throw new ModelException("Unknown OIOBSTSAMLAssertion specification");
		}
    }

	public static OIOBSTSAMLAssertion createOIOBSTCitizenSAMLAssertion(Element assertionElm) {
    	String specVersion = OIOBSTSAMLAssertion.getAttributeValue(assertionElm, OIOSAML3Attributes.SPEC_VERSION, OIOSAMLAttributes.SPEC_VERSION);
		switch (specVersion == null ? "" : specVersion) {
			case "OIO-SAML-3.0": return new OIO3BSTCitizenSAMLAssertion(assertionElm);
			case "DK-SAML-2.0": return new OIO2BSTCitizenSAMLAssertion(assertionElm);
			case "": return new OIOBootstrapToken(assertionElm);
			default: throw new ModelException("Unknown OIOBSTCitizenSAMLAssertion specification");
		}
    }
}
