/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.model;

import dk.sosi.seal.model.constants.DSTags;
import dk.sosi.seal.model.constants.OIOSAMLAttributes;
import dk.sosi.seal.model.constants.SAMLTags;
import dk.sosi.seal.model.constants.SAMLValues;
import dk.sosi.seal.security.HolderOfKey;
import dk.sosi.seal.xml.CertificateParser;
import dk.sosi.seal.xml.XmlUtil;
import org.w3c.dom.Element;

import java.util.Optional;

import static java.util.Optional.*;

/**
 * OIO2BST-LEGACY
 */
public class OIOBootstrapToken extends OIOBSTSAMLAssertion {
    public enum Attribute implements SAMLAttribute {
        // Specifies the version of the OIOSAML profile specification
        SPEC_VER("specVer", OIOSAMLAttributes.SPEC_VERSION, false),
        // Contains the overall level of assurance of the authentication as defined by the Danish [NIST] standard.
        ASSURANCE_LEVEL("assuranceLevel", OIOSAMLAttributes.ASSURANCE_LEVEL, false),
        // Contains the Danish CPR number for the identity represented by 10 digits.
        CPR("cpr", OIOSAMLAttributes.CPR_NUMBER, false);

        private final String id, name;
        private final boolean mandatory;

        Attribute(String id, String name, boolean mandatory) {
            this.id = id; this.name = name; this.mandatory = mandatory;
        }

        public String getId() {
            return id;
        }

        public String getName() {
            return name;
        }

        public String getNameFormat() {
            return SAMLValues.NAME_FORMAT_BASIC;
        }

        public boolean isMandatory() {
            return mandatory;
        }
    }

    public OIOBootstrapToken(Element element) {
        this(element, true);
    }

    public OIOBootstrapToken(Element element, boolean validate) {
        super(element, validate);
    }

    @Override
    public Type getType() {
        return Type.OIO2BST_LEGACY;
    }

    /**
     * Gets the holder of key certificate
     *
     * @return Returns an Option in the 'code>Some</code> state containing the certificate contained in the
     * assertion's XML signature or <code>None</code> if the assertion
     * was not signed
     */
    @Override
    public Optional<HolderOfKey> getHolderOfKeyCertificate() {
        Element certificateElm = getTag(SAMLTags.assertion, SAMLTags.subject, SAMLTags.subjectConfirmation,
                SAMLTags.subjectConfirmationData, DSTags.keyInfo, DSTags.x509Data, DSTags.x509Certificate);
        return certificateElm != null
                ? of(new HolderOfKey(
                CertificateParser.asCertificate(XmlUtil.fromBase64(certificateElm.getTextContent()))))
                : empty();
    }

    @Override
    public String getAssuranceLevel() {
        return getAttributeValue(Attribute.ASSURANCE_LEVEL);
    }

    @Override
    public String getCpr() {
        return getAttributeValue(Attribute.CPR);
    }

    @Override
    public String getSpecVersion() {
        return getAttributeValue(Attribute.SPEC_VER);
    }
}
