/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model.constants;

import dk.sosi.seal.xml.TagUtil;

/**
 * @author chg
 * @since 1.0
 */
public enum DSTags implements Tag {
        x509Certificate("X509Certificate"),
        x509Data("X509Data"),
        x509IssuerSerial("X509IssuerSerial"),
        x509IssuerName("X509IssuerName"),
        x509SerialNumber("X509SerialNumber"),
        keyInfo("KeyInfo"),
        keyName("KeyName"),
        signature("Signature"),
        signatureValue ("SignatureValue"),
        reference("Reference"),
        retrievalMethod("RetrievalMethod"),
        canonicalizationMethod("CanonicalizationMethod");

	
	public static final String X509CERTIFICATE 			= TagUtil.getUnprefixedName(x509Certificate);
	public static final String KEY_INFO 				= TagUtil.getUnprefixedName(keyInfo);
	public static final String SIGNATURE 				= TagUtil.getUnprefixedName(signature);
	public static final String SIGNATURE_VALUE 			= TagUtil.getUnprefixedName(signatureValue);
	public static final String REFERENCE 				= TagUtil.getUnprefixedName(reference);
    public static final String X509DATA 				= TagUtil.getUnprefixedName(x509Data);

	public static final String X509CERTIFICATE_PREFIXED 		= TagUtil.getPrefixedName(x509Certificate);
	public static final String X509DATA_PREFIXED 				= TagUtil.getPrefixedName(x509Data);
	public static final String KEY_INFO_PREFIXED 				= TagUtil.getPrefixedName(keyInfo);
	public static final String KEY_NAME_PREFIXED 				= TagUtil.getPrefixedName(keyName);
	public static final String SIGNATURE_PREFIXED 				= TagUtil.getPrefixedName(signature);
	public static final String RETRIEVAL_METHOD_PREFIXED 		= TagUtil.getPrefixedName(retrievalMethod);
    public static final String CANONICALIZATION_METHOD_PREFIXED	= TagUtil.getPrefixedName(canonicalizationMethod);
    

    @Deprecated
    public static final String X509Certificate = X509CERTIFICATE;
    
    private final String attribute;

    private DSTags(String attribute) {
        this.attribute = attribute;
    }
    
    public String getNS() {
        return NameSpaces.DSIG_SCHEMA;
    }
    public String getPrefix() {
        return NameSpaces.NS_DS;
    }

    public String getTagName() {
        return attribute;
    }
}
