/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model.constants;

import dk.sosi.seal.xml.TagUtil;

/**
 * Class containing SAML XML tags.
 * 
 * @author kkj
 * @since 1.0
 */
public enum SAMLTags implements Tag {
    assertion("Assertion"),
    encryptedAssertion("EncryptedAssertion"),
    conditions("Conditions"), 
    audienceRestriction("AudienceRestriction"),
    attributeStatement("AttributeStatement"),
    attribute("Attribute"),
    attributeValue("AttributeValue"),
    authnStatement("AuthnStatement"),
    authnContext("AuthnContext"),
    authnContextClassRef("AuthnContextClassRef"),
    audience("Audience"),
    subject("Subject"),
    issuer("Issuer"),
    nameID("NameID"),
    subjectConfirmationData("SubjectConfirmationData"),
    subjectConfirmation("SubjectConfirmation"),
    confirmationMethod("ConfirmationMethod")
    ;

    public static final String ASSERTION 					= TagUtil.getUnprefixedName(assertion);
    public static final String ENCRYPTED_ASSERTION 			= TagUtil.getUnprefixedName(encryptedAssertion);
    public static final String ISSUER 						= TagUtil.getUnprefixedName(issuer);
    public static final String CONDITIONS 					= TagUtil.getUnprefixedName(conditions);
    public static final String SUBJECT 						= TagUtil.getUnprefixedName(subject);
    public static final String NAMEID 						= TagUtil.getUnprefixedName(nameID);
    public static final String SUBJECT_CONFIRMATION 		= TagUtil.getUnprefixedName(subjectConfirmation);
    public static final String CONFIRMATION_METHOD 			= TagUtil.getUnprefixedName(confirmationMethod);
    public static final String SUBJECT_CONFIRMATION_DATA	= TagUtil.getUnprefixedName(subjectConfirmationData);
    public static final String ATTRIBUTE_STATEMENT 			= TagUtil.getUnprefixedName(attributeStatement);
    public static final String ATTRIBUTE 					= TagUtil.getUnprefixedName(attribute);
    public static final String ATTRIBUTE_VALUE 				= TagUtil.getUnprefixedName(attributeValue);

    public static final String ASSERTION_PREFIXED 					= TagUtil.getPrefixedName(assertion);
    public static final String ENCRYPTED_ASSERTION_PREFIXED			= TagUtil.getPrefixedName(encryptedAssertion);
    public static final String ISSUER_PREFIXED 						= TagUtil.getPrefixedName(issuer);
    public static final String CONDITIONS_PREFIXED 					= TagUtil.getPrefixedName(conditions);
    public static final String SUBJECT_PREFIXED 					= TagUtil.getPrefixedName(subject);
    public static final String NAMEID_PREFIXED 						= TagUtil.getPrefixedName(nameID);
    public static final String SUBJECT_CONFIRMATION_PREFIXED 		= TagUtil.getPrefixedName(subjectConfirmation);
    public static final String CONFIRMATION_METHOD_PREFIXED 		= TagUtil.getPrefixedName(confirmationMethod);
    public static final String SUBJECT_CONFIRMATION_DATA_PREFIXED	= TagUtil.getPrefixedName(subjectConfirmationData);
    public static final String ATTRIBUTE_STATEMENT_PREFIXED 		= TagUtil.getPrefixedName(attributeStatement);
    public static final String ATTRIBUTE_PREFIXED 					= TagUtil.getPrefixedName(attribute);

    private final String tag;

    private SAMLTags(String tag) {
        this.tag = tag;
    }

    public String getNS() {
        return NameSpaces.SAML2ASSERTION_SCHEMA;
    }

    public String getTagName() {
        return tag;
    }

    public String getPrefix() {
        return NameSpaces.NS_SAML;
    }
}