/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model.constants;

import dk.sosi.seal.xml.TagUtil;

/**
 * Class containing SOAP XML tags.
 * 
 * @author kkj
 * @since 1.0
 */
public enum SOAPTags implements Tag {
        body("Body"),
        header("Header"),
        fault("Fault"), 
        envelope("Envelope"),
        detail("detail"),
        faultactor("faultactor"),
        faultcode("faultcode"),
        faultstring("faultstring");
    
	/*Unprefixed constants - only kept for historic reasons. Should be removed and only Enum values should be used.*/
    public static final String BODY_UNPREFIXED 		= TagUtil.getUnprefixedName(body);
    public static final String HEADER_UNPREFIXED 	= TagUtil.getUnprefixedName(header);
    public static final String ENVELOPE_UNPREFIXED	= TagUtil.getUnprefixedName(envelope);
    public static final String DETAIL 				= TagUtil.getUnprefixedName(detail);
    public static final String FAULTACTOR 			= TagUtil.getUnprefixedName(faultactor);
    public static final String FAULTCODE 			= TagUtil.getUnprefixedName(faultcode);
    public static final String FAULTSTRING 			= TagUtil.getUnprefixedName(faultstring);

	/*Prefixed constants - only kept for historic reasons. Should be removed and only Enum values should be used.*/
    public static final String BODY_PREFIXED 		= TagUtil.getPrefixedName(body);
    public static final String ENVELOPE_PREFIXED	= TagUtil.getPrefixedName(envelope);
    public static final String FAULT_PREFIXED 		= TagUtil.getPrefixedName(fault);
    public static final String HEADER_PREFIXED 		= TagUtil.getPrefixedName(header);
    @Deprecated
    public static final String BODY 				= BODY_PREFIXED;
    @Deprecated
    public static final String HEADER 				= HEADER_PREFIXED;
    @Deprecated
    public static final String ENVELOPE 			= ENVELOPE_PREFIXED;
    @Deprecated
    public static final String FAULT 				= FAULT_PREFIXED;
    
    
    private final String tag;
    
    private SOAPTags(String tag) {
        this.tag = tag;
    }
    
    public String getNS() {
        return NameSpaces.SOAP_SCHEMA;
    }
    public String getTagName() {
        return tag;
    }

    public String getPrefix() {
        return NameSpaces.NS_SOAP;
    }
}