/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model.constants;

import dk.sosi.seal.xml.TagUtil;

/**
 */
public enum WSATags implements Tag {
    action("Action", false),
    address("Address"),
    endpointReference("EndpointReference"),
    relatesTo("RelatesTo"),
    messageID("MessageID", false), //Evt. tilføj wsu til namespaces
    to("To"),
    metadata("Metadata");

	/*Unprefixed constants - only kept for historic reasons. Should be removed and only Enum values should be used.*/
     public static final String MESSAGE_ID 	= TagUtil.getUnprefixedName(messageID);
    public static final String ACTION 		= TagUtil.getUnprefixedName(action);
    public static final String TO 			= TagUtil.getUnprefixedName(to);
    public static final String RELATES_TO	= TagUtil.getUnprefixedName(relatesTo);

    /*Prefixed constants - only kept for historic reasons. Should be removed and only Enum values should be used.*/
    public static final String ACTION_PREFIXED 				= TagUtil.getPrefixedName(action);
    public static final String ADDRESS_PREFIXED 			= TagUtil.getPrefixedName(address);
    public static final String MESSAGE_ID_PREFIXED 			= TagUtil.getPrefixedName(messageID);
    public static final String RELATES_TO_PREFIXED 			= TagUtil.getPrefixedName(relatesTo);
    public static final String TO_PREFIXED 					= TagUtil.getPrefixedName(to);
    public static final String ENDPOINT_REFERENCE_PREFIXED 	= TagUtil.getPrefixedName(endpointReference);

    private final String tag;
    private final boolean useDefaultAttributeId;

    private WSATags(String tag) {
        this(tag, false);
    }
    
    private WSATags(String tag, boolean useIdAttribute) {
        this.tag = tag;
        this.useDefaultAttributeId = useIdAttribute;
    }

    public String getNS() {
        return NameSpaces.WSA_1_0_SCHEMA;
    }

    public String getTagName() {
        return tag;
    }

    public String getPrefix() {
        return NameSpaces.NS_WSA;
    }
    
    @Override
    public boolean useIdAttribute() {
    	return useDefaultAttributeId;
    }
}