/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model.constants;

import dk.sosi.seal.xml.TagUtil;

/**
 * Class containing WSSE XML tags.
 *
 * @author chg
 * @since 1.5.11
 */
public enum WSSETags implements Tag {
	binarySecurityToken("BinarySecurityToken"),
    security("Security", true),
    securityTokenReference("SecurityTokenReference"),
    reference("Reference"),
    transformationParameters("TransformationParameters"),
    usernameToken("UsernameToken"),
    username("Username"),
    password("Password"),
    keyIdentifier("KeyIdentifier");

	/*Unprefixed constants - only kept for historic reasons. Should be removed and only Enum values should be used.*/
    public static String USERNAME_TOKEN 			= TagUtil.getUnprefixedName(usernameToken);
    public static String USERNAME 					= TagUtil.getUnprefixedName(username);
    public static String PASSWORD 					= TagUtil.getUnprefixedName(password);
    public static String SECURITY 					= TagUtil.getUnprefixedName(security);
    public static String SECURITY_TOKEN_REFERENCE 	= TagUtil.getUnprefixedName(securityTokenReference);
    public static String KEY_IDENTIFIER 			= TagUtil.getUnprefixedName(keyIdentifier);
    public static String REFERENCE 			        = TagUtil.getUnprefixedName(reference);
    public static String BINARY_SECURITY_TOKEN      = TagUtil.getUnprefixedName(binarySecurityToken);
    /*Prefixed constants - only kept for historic reasons. Should be removed and only Enum values should be used.*/
    public static String USERNAME_TOKEN_PREFIXED 			= TagUtil.getPrefixedName(usernameToken);
    public static String USERNAME_PREFIXED 					= TagUtil.getPrefixedName(username);
    public static String PASSWORD_PREFIXED 					= TagUtil.getPrefixedName(password);
    public static String SECURITY_PREFIXED 					= TagUtil.getPrefixedName(security);
    public static String SECURITY_TOKEN_REFERENCE_PREFIXED 	= TagUtil.getPrefixedName(securityTokenReference);
    public static String KEY_IDENTIFIER_PREFIXED 			= TagUtil.getPrefixedName(keyIdentifier);
    public static String REFERENCE_PREFIXED 				= TagUtil.getPrefixedName(reference);
    public static String TRANSFORMATION_PARAMETERS_PREFIXED	= TagUtil.getPrefixedName(transformationParameters);

    private final String tagname;
    private final boolean useDefaultAttributeId;


    WSSETags(String tagname) {
        this(tagname, false);
    }
    
    private WSSETags(String tag, boolean useIdAttribute) {
        this.tagname = tag;
        this.useDefaultAttributeId = useIdAttribute;
    }

    public String getNS() {
        return NameSpaces.WSSE_SCHEMA;
    }

    public String getTagName() {
        return tagname;
    }

    public String getPrefix() {
        return NameSpaces.NS_WSSE;
    }
    
    @Override
    public boolean useIdAttribute() {
    	return useDefaultAttributeId;
    }
}
