/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model.dombuilders;

import dk.sosi.seal.model.AbstractOIOBSTSAMLAssertionBuilder;
import dk.sosi.seal.model.EncryptionUtil;
import dk.sosi.seal.model.ModelException;
import dk.sosi.seal.model.OIOBSTSAMLAssertion;
import dk.sosi.seal.model.constants.MedcomAttributes;
import dk.sosi.seal.model.constants.NameSpaces;
import dk.sosi.seal.model.constants.SAMLTags;
import dk.sosi.seal.model.constants.SOSIAttributes;
import dk.sosi.seal.model.constants.WSFAuthAttributes;
import dk.sosi.seal.model.constants.WSFAuthTags;
import dk.sosi.seal.model.constants.WSFAuthValues;
import dk.sosi.seal.model.constants.WSTTags;
import dk.sosi.seal.model.constants.WSTrustAttributes;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.security.PublicKey;

public class OIOBSTSAMLAssertionToIDCardRequestDOMBuilder extends OIOWSTrustRequestDOMBuilder<OIOBSTSAMLAssertionToIDCardRequestDOMBuilder> {
	@Deprecated
	private AbstractOIOBSTSAMLAssertionBuilder<?,?> assertionBuilder;
	private OIOBSTSAMLAssertion assertion;
	
	private PublicKey encryptionKey; 
    private String iTSystemNameClaim, userRoleClaim, userAuthorizationCodeClaim, subjectNameIDClaim;

	/**
	 * @deprecated Use setOIOBSTSAMLAssertion instead
	 */
	@Deprecated
    public OIOBSTSAMLAssertionToIDCardRequestDOMBuilder setOIOSAMLAssertion(AbstractOIOBSTSAMLAssertionBuilder<?,?> builder) {
        this.assertionBuilder = builder;
        return this;
    }

	public OIOBSTSAMLAssertionToIDCardRequestDOMBuilder setOIOBSTSAMLAssertion(OIOBSTSAMLAssertion bst) {
		this.assertion = bst;
		return this;
	}
    
    public OIOBSTSAMLAssertionToIDCardRequestDOMBuilder setEncryptionKey(PublicKey encryptionKey) {
    	this.encryptionKey = encryptionKey;
		return this;
	}
    
    public OIOBSTSAMLAssertionToIDCardRequestDOMBuilder setITSystemName(String iTSystemName) {
    	this.iTSystemNameClaim = iTSystemName;
		return this;
	}
    
    public OIOBSTSAMLAssertionToIDCardRequestDOMBuilder setUserRole(String userRole) {
    	this.userRoleClaim = userRole;
		return this;
	}
    
    public OIOBSTSAMLAssertionToIDCardRequestDOMBuilder setUserAuthorizationCode(String userAuthorizationCode) {
    	this.userAuthorizationCodeClaim = userAuthorizationCode;
		return this;
	}
    
    public OIOBSTSAMLAssertionToIDCardRequestDOMBuilder setSubjectNameID(String subjectNameID) {
    	this.subjectNameIDClaim = subjectNameID;
		return this;
	}
    
    @Override
    protected void validateBeforeBuild() throws ModelException {
    	super.validateBeforeBuild();
    	validate("signingVault", signingVault);
		// Should be removed when deprecated method setOIOSAMLAssertion is removed
		if (assertionBuilder != null) {
			assertion = validateAndBuildAssertion();
		}
		validate("OIOBSTSAMLAssertion", assertion);
		validate("ITSystemName", iTSystemNameClaim);
    }

    protected OIOBSTSAMLAssertion validateAndBuildAssertion() {
        assertionBuilder.validateBeforeBuild();
        return assertionBuilder.build();
    }
	
	@Override
	protected void addRootAttributes(Element envelope) {
		super.addRootAttributes(envelope);
		addNS(envelope, NameSpaces.NS_WSF_AUTH	, NameSpaces.WSF_AUTH_SCHEMA);
		addNS(envelope, NameSpaces.NS_DS		, NameSpaces.DSIG_SCHEMA);
		addNS(envelope, NameSpaces.NS_WSSE		, NameSpaces.WSSE_SCHEMA);
		addNS(envelope, NameSpaces.NS_WSU		, NameSpaces.WSU_SCHEMA);
	}

	@Override
	protected void addActAsTokens(Document doc, Element actAs) {
		if(encryptionKey != null) {
			Document assertionDoc = assertion.getDOM();
			Element assertionElm = assertionDoc.getDocumentElement();
			Element encryptedAssertionElm = assertionDoc.createElementNS(NameSpaces.SAML2ASSERTION_SCHEMA, SAMLTags.ENCRYPTED_ASSERTION_PREFIXED);
			encryptedAssertionElm.appendChild(assertionElm);
			EncryptionUtil.encrypt(assertionElm, encryptionKey);
			actAs.appendChild(doc.importNode(encryptedAssertionElm, true));	
		} else {
			actAs.appendChild(doc.importNode(assertion.getDOM().getDocumentElement(), true));
		}
	}
   
	@Override
    protected void addClaims(Document doc, Element requestSecurityToken) {
        Element claimsElm = createElement(WSTTags.claims);
        claimsElm.setAttributeNS(null, WSTrustAttributes.DIALECT, WSFAuthValues.CLAIMS_DIALECT);
        requestSecurityToken.appendChild(claimsElm);

        Element cprClaimElm = buildClaimElement(MedcomAttributes.IT_SYSTEM_NAME, this.iTSystemNameClaim);
        claimsElm.appendChild(cprClaimElm);
        
		if (this.userRoleClaim != null) {
			Element userRoleClaimElm = buildClaimElement(MedcomAttributes.USER_ROLE, this.userRoleClaim);
			claimsElm.appendChild(userRoleClaimElm);
		}
		
		if (this.userAuthorizationCodeClaim != null) {
			Element userAuthorizationCodeClaimElm = buildClaimElement(MedcomAttributes.USER_AUTHORIZATION_CODE, this.userAuthorizationCodeClaim);
			claimsElm.appendChild(userAuthorizationCodeClaimElm);
		}
		
		if (this.subjectNameIDClaim != null) {
			Element subjectNameIDClaimElm = buildClaimElement(SOSIAttributes.SUBJECT_NAME_ID, this.subjectNameIDClaim);
			claimsElm.appendChild(subjectNameIDClaimElm);
		}
    }
	
    private Element buildClaimElement(String attributeName, String value) {
        Element cprClaimElm = createElement(WSFAuthTags.claimType);
        cprClaimElm.setAttributeNS(null, WSFAuthAttributes.URI, attributeName);

        Element valueElm = createElement(WSFAuthTags.value);
        valueElm.setTextContent(value);
        cprClaimElm.appendChild(valueElm);
        return cprClaimElm;
    }
}
