/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.model.dombuilders;

import dk.sosi.seal.model.EncryptionUtil;
import dk.sosi.seal.model.OIOBSTSAMLAssertion;
import dk.sosi.seal.model.constants.NameSpaces;
import dk.sosi.seal.model.constants.SAMLTags;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import dk.sosi.seal.model.ModelException;

import java.security.PublicKey;

/**
 */
public class OIOBootstrapToIdentityTokenRequestDOMBuilder extends AbstractOIOToIdentityTokenRequestDOMBuilder<OIOBootstrapToIdentityTokenRequestDOMBuilder>{

    private OIOBSTSAMLAssertion bootstrapToken;
    private PublicKey encryptionKey;

    public OIOBootstrapToIdentityTokenRequestDOMBuilder setOIOBootstrapToken(OIOBSTSAMLAssertion bootstrapToken) {
        this.bootstrapToken = bootstrapToken;
        return this;
    }

    public OIOBootstrapToIdentityTokenRequestDOMBuilder setEncryptionKey(PublicKey encryptionKey) {
        this.encryptionKey = encryptionKey;
        return this;
    }

    @Override
    protected void validateBeforeBuild() throws ModelException {
        super.validateBeforeBuild();
        validate("CPRNumberClaim", cprNumberClaim);
        validate("OIOBootstrapToken", bootstrapToken);
    }

    @Override
    protected void addActAsTokens(Document doc, Element actAs) {
        Document assertionDocument = bootstrapToken.getDOM();
        Element assertionElement = assertionDocument.getDocumentElement();

        if(encryptionKey != null) {
            Element encryptedAssertionElm = assertionDocument.createElementNS(NameSpaces.SAML2ASSERTION_SCHEMA, SAMLTags.ENCRYPTED_ASSERTION_PREFIXED);
            encryptedAssertionElm.appendChild(assertionElement);
            EncryptionUtil.encrypt(assertionElement, encryptionKey);
            actAs.appendChild(doc.importNode(encryptedAssertionElm, true));
        } else {
            actAs.appendChild(doc.importNode(assertionElement, true));
        }
    }

}
