/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.pki.internal;

import dk.sosi.seal.pki.DistinguishedName;

/**
 */
public class SOSISTSCertificateMatcher {

    private final static String FOCES2_STS_SUBJECT_SERIALNUMBER_PREFIX = "CVR:33257872-FID:";
    private final static String FOCES3_STS_SUBJECT_ORGANIZATION_IDENTIFIER = "NTRDK-33257872";

    private final String newFocesSTSCertPrefix;

    /**
     * @deprecated use {@link #SOSISTSCertificateMatcher(String)}
     */
    @Deprecated
    public SOSISTSCertificateMatcher(String oldSTSCertSSN, String newFocesSTSCertPrefix) {
        this.newFocesSTSCertPrefix = newFocesSTSCertPrefix;
    }

    public SOSISTSCertificateMatcher(String newFocesSTSCertPrefix) {
        this.newFocesSTSCertPrefix = newFocesSTSCertPrefix;
    }

    public boolean matches(DistinguishedName subjectDN) {
        return subjectDN.getCommonName() != null && subjectDN.getCommonName().startsWith(newFocesSTSCertPrefix)
                && (subjectDN.getSubjectSerialNumber() != null && subjectDN.getSubjectSerialNumber().startsWith(FOCES2_STS_SUBJECT_SERIALNUMBER_PREFIX) ||
                   (subjectDN.getOrganizationIdentifier() != null && subjectDN.getOrganizationIdentifier().contains(FOCES3_STS_SUBJECT_ORGANIZATION_IDENTIFIER)));
    }
}
