/**
 * The MIT License
 * <p>
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 * <p>
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * <p>
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * <p>
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.seal.security;

import dk.sosi.seal.common.functionalExt.either.Either;
import org.jcp.xml.dsig.internal.dom.DOMX509IssuerSerial;

import javax.xml.crypto.dsig.keyinfo.X509IssuerSerial;
import java.math.BigInteger;
import java.security.cert.X509Certificate;
import java.util.function.Function;

import static dk.sosi.seal.common.functionalExt.either.Either.*;

public class HolderOfKey {
    public final Either<X509Certificate, X509IssuerSerial> certificate;

    public HolderOfKey(X509Certificate certificate) {this.certificate = left(certificate);}

    public HolderOfKey(X509IssuerSerial certificate) {this.certificate = right(certificate);}

    public HolderOfKey(String issuerName, BigInteger issuerSerial) {
        certificate = right(new DOMX509IssuerSerial(issuerName, issuerSerial));
    }

    public boolean matches(X509Certificate c) {return certificate.match(c::equals, compareIssuerWithHok.apply(c));}

    private final static Function<X509Certificate, Function<X509IssuerSerial, Boolean>> compareIssuerWithHok =
            c -> issuerSerial -> c.getSerialNumber().equals(issuerSerial.getSerialNumber())
                    && c.getIssuerDN().getName().equals(issuerSerial.getIssuerName());
}
