/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal;

import dk.sosi.seal.model.OIOBootstrapToIdentityTokenRequest;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.CredentialVaultTestUtil;
import dk.sosi.seal.xml.XmlUtil;
import org.junit.Before;
import org.junit.Test;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

public class OIOSAMLFactoryReferenceTest {
	public CredentialVault untrusted = CredentialVaultTestUtil.getVoces3CredentialVault();
	private OIOSAMLFactory factory;

	@Before
	public void setUp() {
		factory = new OIOSAMLFactory();
	}

	@Test
	public void testJavaReferenceImplementation() {
		InputSource inputSource = new InputSource(this.getClass().getResourceAsStream("/oiosaml-examples/StsRequestJavaReference.xml"));
		Document document = XmlUtil.readXml(System.getProperties(), inputSource, false);

		// write to string and parse to XML
		Document requestDoc = XmlUtil.readXml(System.getProperties(), XmlUtil.node2String(document), false);

		OIOBootstrapToIdentityTokenRequest assertionRequest = factory.createOIOBootstrapToIdentityTokenRequestModelBuilder().build(requestDoc);
		assertionRequest.validateSignature(); // Throws exception if it does not validate.
		assertionRequest.validateHolderOfKeyRelation();
	}

	@Test
	public void testNetReferenceImplementation() {
		InputSource inputSource = new InputSource(this.getClass().getResourceAsStream("/oiosaml-examples/StsRequestNetReference.xml"));
		Document document = XmlUtil.readXml(System.getProperties(), inputSource, false);

		// write to string and parse to XML
		Document requestDoc = XmlUtil.readXml(System.getProperties(), XmlUtil.node2String(document), false);

		OIOBootstrapToIdentityTokenRequest assertionRequest = factory.createOIOBootstrapToIdentityTokenRequestModelBuilder().build(requestDoc);
		assertionRequest.validateSignature(); // Throws exception if it does not validate.
		assertionRequest.validateHolderOfKeyRelation();
	}
}
