/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.model;

import java.util.Properties;

import dk.sosi.seal.pki.CertificationAuthority;
import dk.sosi.seal.pki.CertificationAuthorityFactory;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.pki.MockFederation;
import dk.sosi.seal.pki.NaiveCertificateStatusChecker;
import dk.sosi.seal.vault.CredentialVaultTestUtil;

public abstract class AbstractIdentityTokenTest extends AbstractUserIDCardTest {
    protected IdentityTokenBuilder createBuilder() {
        UserIDCard uidc = createUserIDCard();

        IdentityTokenBuilder itb = new IdentityTokenBuilder(sosiFactory.getCredentialVault());
        itb.setAudienceRestriction("http://fmk-online.dk");
        itb.setNotBefore(notBefore);
        itb.setNotOnOrAfter(notOnOrAfter);
        itb.setUserIdCard(uidc);
        itb.requireOrganizationName();
        itb.requireCvrNumberIdentifier();
        itb.requireCprNumber();
        itb.requireITSystemName();
        itb.requireUserAuthorizationCode();
        itb.requireUserEducationCode();
        itb.setIssuer("http://pan.certifikat.dk/sts/services/SecurityTokenService");
        return itb;
    }
    
    /**
     * Creates a <code>Federation</code> for test purposes.
     * 
     * @return A test <code>Federation</code> instance.
     */
    protected Federation getMockFederation() {
        Properties properties = System.getProperties();
        final CertificationAuthority ca = CertificationAuthorityFactory.create(properties, CertificationAuthorityFactory.OCES_SYSTEMTEST_CA,
                new NaiveCertificateStatusChecker(properties), CredentialVaultTestUtil.getCertificateCacheForVoces3CredentialVaultCert());
        return new MockFederation(properties, ca, "UI:DK-O:G:c70b0207-162e-4d3d-a7f1-a19a8e07d99b");
    }
}
