package dk.sosi.seal.model;

import static org.junit.Assert.assertEquals;

import org.junit.Test;
import org.w3c.dom.Document;

import dk.sosi.seal.model.BasicPrivileges.Constraint;
import dk.sosi.seal.model.BasicPrivileges.Profile;
import dk.sosi.seal.xml.XmlUtil;

public class BasicPrivilegesDOMBuilderTest {
	
	private static final Constraint constraint = new Constraint("constraint", "constraint_value");
	
	@Test(expected=ModelException.class)
	public void testNoPrivilegesDefined() {		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder();
		builder.build();
	}
	
	@Test(expected=ModelException.class)
	public void testOnlyConstraintDefined() {		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder(Profile.V12);
		builder.addConstraint("scope", constraint);
		builder.build();
	}
	
	@Test(expected=ModelException.class)
	public void testPrivilegeGroupMissingSope() {		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder();
		builder.addPrivilege(null, null);
		builder.build();
	}
	
	@Test(expected=ModelException.class)
	public void testBlankPrivilegeForScope() {		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder();
		builder.addPrivilege("scope", null);
		builder.build();
	}
	
	@Test
	public void testOneScopeOnePrivilege() {		
		String expectedXml = 
				"<bpp:PrivilegeList xmlns:bpp=\"http://itst.dk/oiosaml/basic_privilege_profile\">\n" + 
				"  <bpp:PrivilegeGroup Scope=\"scope\">\n" + 
				"    <bpp:Privilege>privilege</bpp:Privilege>\n" + 
				"  </bpp:PrivilegeGroup>\n" + 
				"</bpp:PrivilegeList>\n";
		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder();
		builder.addPrivilege("scope", "privilege");
		Document build = builder.build();
		
		String actualXml = XmlUtil.node2String(build, true, false);
		assertEquals(expectedXml, actualXml);
	}
	
	@Test
	public void testOneScopeOnePrivilege12() {		
		String expectedXml = 
				"<bpp:PrivilegeList xmlns:bpp=\"http://digst.dk/oiosaml/basic_privilege_profile\">\n" + 
				"  <PrivilegeGroup Scope=\"scope\">\n" + 
				"    <Privilege>privilege</Privilege>\n" + 
				"  </PrivilegeGroup>\n" + 
				"</bpp:PrivilegeList>\n";
		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder(Profile.V12);
		builder.addPrivilege("scope", "privilege");
		Document build = builder.build();
		
		String actualXml = XmlUtil.node2String(build, true, false);
		assertEquals(expectedXml, actualXml);
	}
	
	@Test
	public void testOneScopeOnePrivilegeOneConstraint12() {		
		String expectedXml = 
				"<bpp:PrivilegeList xmlns:bpp=\"http://digst.dk/oiosaml/basic_privilege_profile\">\n" + 
				"  <PrivilegeGroup Scope=\"scope\">\n" + 
				"    <Privilege>privilege</Privilege>\n" + 
				"    <Constraint Name=\"constraint\">constraint_value</Constraint>\n" + 
				"  </PrivilegeGroup>\n" + 
				"</bpp:PrivilegeList>\n";
		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder(Profile.V12);
		builder.addPrivilege("scope", "privilege");
		builder.addConstraint("scope", constraint);
		Document build = builder.build();
		
		String actualXml = XmlUtil.node2String(build, true, false);
		assertEquals(expectedXml, actualXml);
	}
	
	@Test(expected=ModelException.class)
	public void testFirstScopeOnePrivilegeSecondScopeOneConstraint12() {		
		BasicPrivilegesDOMBuilder builder = new BasicPrivilegesDOMBuilder(Profile.V12);
		builder.addPrivilege("first", "privilege");
		builder.addConstraint("second", constraint);
		builder.build();
	}

}
