package dk.sosi.seal.model;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import dk.sosi.seal.model.BasicPrivileges.Profile;
import dk.sosi.seal.model.constants.BasicPrivileges12Tags;
import dk.sosi.seal.model.constants.SOAPTags;
import dk.sosi.seal.xml.TagUtil;
import dk.sosi.seal.xml.XmlUtil;

public class BasicPriviliges12Test {
	
	@Test(expected=NullPointerException.class)
	public void testNull() {
		Element elm = null;
		new BasicPrivileges(elm, Profile.V12);
	}
	
	@Test(expected=ModelException.class)
	public void testNoScopeNoPriviligeInDocument() {
		String soapXml = 				
				"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" + 
				"<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" + 
				"  <soapenv:Header />\n" + 			 
				"  <soapenv:Body />\n" +
				"</soapenv:Envelope>";
		
		Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
		Element body = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body);
		
		new BasicPrivileges(body, Profile.V12);
	}
	
	@Test(expected=ModelException.class)
	public void testOneScopeNoPriviligeInDocument() {
		String soapXml = 
				"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" + 
				"<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" + 
				"  <soapenv:Header />\n" + 			 
				"  <soapenv:Body>\n" +
				"    <bpp:PrivilegeList xmlns:bpp=\"http://digst.dk/oiosaml/basic_privilege_profile\">\n" + 
				"      <PrivilegeGroup Scope=\"MyScope\" />\n" + 
				"    </bpp:PrivilegeList>\n" +
				"  </soapenv:Body>\n" +			
				"</soapenv:Envelope>";
		
		Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
		Element privilegeList = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BasicPrivileges12Tags.PRIVILEGE_LIST);
		
		new BasicPrivileges(privilegeList, Profile.V12);
	}
	
	@Test
	public void testOneScopeOnePriviligeInDocument() {
		String soapXml = 
				"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" + 
				"<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" + 
				"  <soapenv:Header />\n" + 
				"  <soapenv:Body>\n" + 
				"    <bpp:PrivilegeList xmlns:bpp=\"http://digst.dk/oiosaml/basic_privilege_profile\">\n" + 
				"      <PrivilegeGroup Scope=\"*scope*\">\n" + 
				"        <Privilege>*privilige*</Privilege>\n" + 
				"      </PrivilegeGroup>\n" + 
				"    </bpp:PrivilegeList>\n" + 
				"  </soapenv:Body>\n" + 
				"</soapenv:Envelope>";
		
		Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
		Element privilegeList = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BasicPrivileges12Tags.PRIVILEGE_LIST);
		
		BasicPrivileges basicPrivileges = new BasicPrivileges(privilegeList, Profile.V12);
		
		Set<String> scopes = basicPrivileges.getScopes();
		assertEquals(1, scopes.size());
		assertTrue(scopes.contains("*scope*"));
		
		List<String> privileges = basicPrivileges.getPrivileges("*scope*");
		assertEquals(1, privileges.size());
		assertTrue(privileges.contains("*privilige*"));
		
	}
}
