package dk.sosi.seal.model;

import dk.sosi.seal.model.constants.BlurringInstructions11Tags;
import dk.sosi.seal.model.constants.SOAPTags;
import dk.sosi.seal.xml.TagUtil;
import dk.sosi.seal.xml.XmlUtil;
import org.junit.Assert;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.List;
import java.util.Properties;

import static dk.sosi.seal.model.BlurredOrganisation.OrgType.Cvr;
import static dk.sosi.seal.model.BlurredOrganisation.Reason.FromRelatedPerson;
import static dk.sosi.seal.model.BlurredOrganisation.Reason.SpecificForPerson;

public class BlurringInstructions11Test {

    @Test(expected = NullPointerException.class)
    public void testNull() {
        Element elm = null;
        new BlurringInstructions11Impl(elm);
    }

    @Test(expected = ModelException.class)
    public void testNoSaltInDocument() {
        String soapXml =
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                        "  <soapenv:Header />\n" +
                        "  <soapenv:Body>\n" +
                        "     <bip:BlurringInstructions xmlns:bip=\"urn:dk:healthcare:saml:blurring_instruction_profile:1.1\">" +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"CVR\" reason=\"specific_for_person\">29190925</bip:BlurEmployeeNamesFromOrg>" +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"CVR\" reason=\"from_related_person\">29190941</bip:BlurEmployeeNamesFromOrg> " +
                        "     </bip:BlurringInstructions>" +
                        "  </soapenv:Body>\n" +
                        "</soapenv:Envelope>";

        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element blurringInstructions = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BlurringInstructions11Tags.BLURRING_INSTRUCTIONS);
        new BlurringInstructions11Impl(blurringInstructions);
    }

    @Test(expected = ModelException.class)
    public void testWrongOrgTypeInDocument() {
        String soapXml =
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                        "  <soapenv:Header />\n" +
                        "  <soapenv:Body>\n" +
                        "     <bip:BlurringInstructions xmlns:bip=\"urn:dk:healthcare:saml:blurring_instruction_profile:1.1\" currentSalt=\"5kZZLNQMNIkz1Y7tCDj3GQ==\">" +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"KUK\" reason=\"specific_for_person\">29190925</bip:BlurEmployeeNamesFromOrg>" +
                        "     </bip:BlurringInstructions>" +
                        "  </soapenv:Body>\n" +
                        "</soapenv:Envelope>";

        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element blurringInstructions = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BlurringInstructions11Tags.BLURRING_INSTRUCTIONS);

        new BlurringInstructions11Impl(blurringInstructions);
    }

    @Test(expected = ModelException.class)
    public void testWrongReasonInDocument() {
        String soapXml =
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                        "  <soapenv:Header />\n" +
                        "  <soapenv:Body>\n" +
                        "     <bip:BlurringInstructions xmlns:bip=\"urn:dk:healthcare:saml:blurring_instruction_profile:1.1\" currentSalt=\"5kZZLNQMNIkz1Y7tCDj3GQ==\">" +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"CVR\" reason=\"kuk_kuk\">29190925</bip:BlurEmployeeNamesFromOrg>" +
                        "     </bip:BlurringInstructions>" +
                        "  </soapenv:Body>\n" +
                        "</soapenv:Envelope>";

        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element blurringInstructions = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BlurringInstructions11Tags.BLURRING_INSTRUCTIONS);

        new BlurringInstructions11Impl(blurringInstructions);
    }

    @Test
    public void testZeroBlurredOrganisationInDocument() {
        String soapXml =
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                        "  <soapenv:Header />\n" +
                        "  <soapenv:Body>\n" +
                        "     <bip:BlurringInstructions xmlns:bip=\"urn:dk:healthcare:saml:blurring_instruction_profile:1.1\" currentSalt=\"5kZZLNQMNIkz1Y7tCDj3GQ==\"/>" +
                        "  </soapenv:Body>\n" +
                        "</soapenv:Envelope>";

        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element blurringInstructions = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BlurringInstructions11Tags.BLURRING_INSTRUCTIONS);

        BlurringInstructions11Impl blurringInstructions11 = new BlurringInstructions11Impl(blurringInstructions);

        List<BlurredOrganisation> orgs = blurringInstructions11.getBlurredOrganisations();
        Assert.assertEquals(0, orgs.size());
    }

    @Test
    public void testThreeBlurredOrganisationsInDocument() {
        String soapXml =
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                        "  <soapenv:Header />\n" +
                        "  <soapenv:Body>\n" +
                        "     <bip:BlurringInstructions xmlns:bip=\"urn:dk:healthcare:saml:blurring_instruction_profile:1.1\"  currentSalt=\"5kZZLNQMNIkz1Y7tCDj3GQ==\">" +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"CVR\" reason=\"specific_for_person\">29190925</bip:BlurEmployeeNamesFromOrg>" +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"CVR\" reason=\"from_related_person\">29190941</bip:BlurEmployeeNamesFromOrg> " +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"SOR\" reason=\"specific_department\">12345678</bip:BlurEmployeeNamesFromOrg> " +
                        "     </bip:BlurringInstructions>" +
                        "  </soapenv:Body>\n" +
                        "</soapenv:Envelope>";

        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element blurringInstructions = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BlurringInstructions11Tags.BLURRING_INSTRUCTIONS);

        BlurringInstructions11Impl blurringInstructions11 = new BlurringInstructions11Impl(blurringInstructions);

        List<BlurredOrganisation> orgs = blurringInstructions11.getBlurredOrganisations();
        Assert.assertEquals(3, orgs.size());

        Assert.assertEquals("29190925", orgs.get(0).getValue());
        Assert.assertEquals(Cvr, orgs.get(0).getOrgType());
        Assert.assertEquals(SpecificForPerson, orgs.get(0).getReason());

        Assert.assertEquals("29190941", orgs.get(1).getValue());
        Assert.assertEquals(Cvr, orgs.get(1).getOrgType());
        Assert.assertEquals(FromRelatedPerson, orgs.get(1).getReason());

        Assert.assertEquals("12345678", orgs.get(2).getValue());
        Assert.assertEquals(BlurredOrganisation.OrgType.Sor, orgs.get(2).getOrgType());
        Assert.assertEquals(BlurredOrganisation.Reason.SpecificDepartment, orgs.get(2).getReason());
    }

    @Test
    public void testShakForSpecificDepartment() {
        String soapXml =
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                        "  <soapenv:Header />\n" +
                        "  <soapenv:Body>\n" +
                        "     <bip:BlurringInstructions xmlns:bip=\"urn:dk:healthcare:saml:blurring_instruction_profile:1.1\"  currentSalt=\"5kZZLNQMNIkz1Y7tCDj3GQ==\">" +
                        "        <bip:BlurEmployeeNamesFromOrg orgType=\"SHAK\" reason=\"specific_department\">12345678</bip:BlurEmployeeNamesFromOrg> " +
                        "     </bip:BlurringInstructions>" +
                        "  </soapenv:Body>\n" +
                        "</soapenv:Envelope>";

        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element blurringInstructions = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, BlurringInstructions11Tags.BLURRING_INSTRUCTIONS);

        BlurringInstructions11Impl blurringInstructions11 = new BlurringInstructions11Impl(blurringInstructions);

        List<BlurredOrganisation> orgs = blurringInstructions11.getBlurredOrganisations();
        Assert.assertEquals(1, orgs.size());

        Assert.assertEquals("12345678", orgs.get(0).getValue());
        Assert.assertEquals(BlurredOrganisation.OrgType.Shak, orgs.get(0).getOrgType());
        Assert.assertEquals(BlurredOrganisation.Reason.SpecificDepartment, orgs.get(0).getReason());
    }
}