/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.model;

import dk.sosi.seal.model.dombuilders.HealthcareContextTokenDOMBuilder;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 */
public class HealthcareContextTokenBuilderTest {

    @Rule
    public ExpectedException expectedException = ExpectedException.none();

    @Test
    public void checkSetNullAttributeName() {
        expectedException.expect(IllegalArgumentException.class);
        expectedException.expectMessage("attributeName cannot be null or empty");
        new HealthcareContextTokenDOMBuilder().setAttribute(null, null);
    }

    @Test
    public void checkSetEmptyAttributeName() {
        expectedException.expect(IllegalArgumentException.class);
        expectedException.expectMessage("attributeName cannot be null or empty");
        new HealthcareContextTokenDOMBuilder().setAttribute("", null);
    }

    @Test
    public void checkSetNullAttributeValue() {
        expectedException.expect(IllegalArgumentException.class);
        expectedException.expectMessage("attributeValue for 'Foo' cannot be null or empty");
        new HealthcareContextTokenDOMBuilder().setAttribute("Foo", null);
    }

    @Test
    public void checkSetEmptyAttributeValue() {
        expectedException.expect(IllegalArgumentException.class);
        expectedException.expectMessage("attributeValue for 'Foo' cannot be null or empty");
        new HealthcareContextTokenDOMBuilder().setAttribute("Foo", "   ");
    }

}
