package dk.sosi.seal.model;

import dk.sosi.seal.model.constants.SOAPTags;
import dk.sosi.seal.model.constants.SubjectRelations11Tags;
import dk.sosi.seal.xml.TagUtil;
import dk.sosi.seal.xml.XmlUtil;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.Properties;

import static org.junit.Assert.*;

public class SubjectRelations11Test {

    @Test
    public void subjectRelations11DomBuilder_parentalCustodyHolder_containsCorrectValues() {
        // Arrange
        String type = "parentalCustodyHolder";
        String cpr = "0303033333";
        int age = 25;
        String soapXml = getSoapXml(cpr, type, age);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations11Tags.SUBJECT_RELATIONS);
        SubjectRelations11Impl subjectRelations11Impl = new SubjectRelations11Impl(elm);

        // Act
        SubjectRelations11DomBuilder subjectRelations11DomBuilder = new SubjectRelations11DomBuilder(subjectRelations11Impl);
        Document build = subjectRelations11DomBuilder.build();

        // Assert
        String actualXml = XmlUtil.node2String(build, true, false);
        assertNotNull(actualXml);
        assertTrue(actualXml.contains(type));
        assertTrue(actualXml.contains(cpr));
        assertEquals(age, 25);
    }

    @Test
    public void subjectRelations11DomBuilder_partlyWardCustodyHolder_containsCorrectValues() {
        // Arrange
        String type = "partlyWardCustodyHolder";
        String cpr = "0303034444";
        int age = 25;
        String soapXml = getSoapXml(cpr, type, age);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations11Tags.SUBJECT_RELATIONS);
        SubjectRelations11Impl subjectRelations11Impl = new SubjectRelations11Impl(elm);

        // Act
        SubjectRelations11DomBuilder subjectRelations11DomBuilder = new SubjectRelations11DomBuilder(subjectRelations11Impl);
        Document build = subjectRelations11DomBuilder.build();

        // Assert
        String actualXml = XmlUtil.node2String(build, true, false);
        assertNotNull(actualXml);
        assertTrue(actualXml.contains(type));
        assertTrue(actualXml.contains(cpr));
        assertEquals(age, 25);
    }

    @Test
    public void subjectRelations11DomBuilder_wardCustodyHolder_containsCorrectValues() {
        // Arrange
        String type = "wardCustodyHolder";
        String cpr = "0303035555";
        int age = 25;
        String soapXml = getSoapXml(cpr, type, age);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations11Tags.SUBJECT_RELATIONS);
        SubjectRelations11Impl subjectRelations11Impl = new SubjectRelations11Impl(elm);

        // Act
        SubjectRelations11DomBuilder subjectRelations11DomBuilder = new SubjectRelations11DomBuilder(subjectRelations11Impl);
        Document build = subjectRelations11DomBuilder.build();

        // Assert
        String actualXml = XmlUtil.node2String(build, true, false);
        assertNotNull(actualXml);
        assertTrue(actualXml.contains(type));
        assertTrue(actualXml.contains(cpr));
        assertEquals(age, 25);
    }

    @Test
    public void subjectRelations11DomBuilder_noSubjectRelationAge_noErrors() {
        // Arrange
        String type = "wardCustodyHolder";
        String cpr = "0303035555";
        String soapXml = getSoapXmlWithoutAge(cpr, type);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations11Tags.SUBJECT_RELATIONS);
        SubjectRelations11Impl subjectRelations11Impl = new SubjectRelations11Impl(elm);

        // Act
        SubjectRelations11DomBuilder subjectRelations11DomBuilder = new SubjectRelations11DomBuilder(subjectRelations11Impl);
        Document build = subjectRelations11DomBuilder.build();

        // Assert
        String actualXml = XmlUtil.node2String(build, true, false);
        assertNotNull(actualXml);
        assertTrue(actualXml.contains(type));
        assertTrue(actualXml.contains(cpr));
    }

    @Test
    public void subjectRelations11Impl_ageNotNull_containsCorrectValues() {
        // Arrange
        String type = "wardCustodyHolder";
        String cpr = "0303036666";
        int age = 25;
        String soapXml = getSoapXml(cpr, type, age);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations11Tags.SUBJECT_RELATIONS);

        // Act
        SubjectRelations11Impl subjectRelations11Impl = new SubjectRelations11Impl(elm);

        // Assert
        assertEquals(1, subjectRelations11Impl.getVerifiedRelations().size());
        SubjectRelations11Impl.VerifiedRelation verifiedRelation = subjectRelations11Impl.getVerifiedRelations().get(0);
        assertNotNull(verifiedRelation.getRelatedPersonAge());
        assertEquals(age, verifiedRelation.getRelatedPersonAge().get().intValue());
    }

    @Test
    public void subjectRelations11Impl_ageRelationWithoutAge_containsCorrectValues() {
        // Arrange
        String type = "wardCustodyHolder";
        String cpr = "0303036666";
        String soapXml = getSoapXmlWithoutAge(cpr, type);
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations11Tags.SUBJECT_RELATIONS);

        // Act
        SubjectRelations11Impl subjectRelations11Impl = new SubjectRelations11Impl(elm);

        // Assert
        assertEquals(1, subjectRelations11Impl.getVerifiedRelations().size());
        SubjectRelations11Impl.VerifiedRelation verifiedRelation = subjectRelations11Impl.getVerifiedRelations().get(0);
        assertFalse(verifiedRelation.getRelatedPersonAge().isPresent());
    }

    @Test
    public void subjectRelations11Impl_emptyVerifiedRelation_validationError() {
        // Arrange
        String soapXml = getEmptySoapXml();
        Document doc = XmlUtil.readXml(new Properties(), soapXml, true);
        Element elm = TagUtil.getFirstChildElementNS(doc.getDocumentElement(), SOAPTags.body, SubjectRelations11Tags.SUBJECT_RELATIONS);

        // Act
        try {
            new SubjectRelations11Impl(elm);
            fail("Should have thrown exception");
        } catch (ModelException e) {
            // Assert
            assertTrue(e.getMessage().contains("Error validating subject relations"));
        }
    }

    private static String getEmptySoapXml() {
        return  "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                "  <soapenv:Header />\n" +
                "  <soapenv:Body>\n" +
                "    <srp:SubjectRelations xmlns:srp=\"urn:dk:healthcare:saml:subject_relations_profile:1.1\">\n" +
                "    </srp:SubjectRelations>\n" +
                "  </soapenv:Body>\n" +
                "</soapenv:Envelope>";
    }

    private static String getSoapXml(String personId, String relatedType, int age) {
        final String subjectRelationsXml =
                    "<srp:SubjectRelations xmlns:srp=\"urn:dk:healthcare:saml:subject_relations_profile:1.1\">\n" +
                    "   <srp:VerifiedRelation\n" +
                    "      relationType=\""+relatedType+"\"\n" +
                    "      relatedPersonID=\""+personId+"\"\n" +
                    "      relatedPersonIDType=\"URN:OID:1.2.208.176.1.2\"\n" +
                    "      relatedPersonAge=\""+age+"\"/>\n" +
                    "</srp:SubjectRelations>\n";

        return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                    "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                    "  <soapenv:Header />\n" +
                    "  <soapenv:Body>\n" +
                        subjectRelationsXml +
                    "  </soapenv:Body>\n" +
                    "</soapenv:Envelope>";

    }

    private static String getSoapXmlWithoutAge(String personId, String relatedType) {
        final String subjectRelationsXml =
                    "<srp:SubjectRelations xmlns:srp=\"urn:dk:healthcare:saml:subject_relations_profile:1.1\">\n" +
                    "   <srp:VerifiedRelation\n" +
                    "      relationType=\""+relatedType+"\"\n" +
                    "      relatedPersonID=\""+personId+"\"\n" +
                    "      relatedPersonIDType=\"URN:OID:1.2.208.176.1.2\"\n" +
                    "   />\n" +
                    "</srp:SubjectRelations>\n";

        return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
                    "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                    "  <soapenv:Header />\n" +
                    "  <soapenv:Body>\n" +
                        subjectRelationsXml +
                    "  </soapenv:Body>\n" +
                    "</soapenv:Envelope>";

    }

}