package dk.sosi.seal.model.mitid.builder;

import dk.sosi.seal.OIOSAMLFactory;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.*;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import dk.sosi.seal.model.dombuilders.DOMBuilderHelper;
import dk.sosi.seal.model.dombuilders.OIOBSTSAMLAssertionToIDCardResponseDOMBuilder;
import dk.sosi.seal.modelbuilders.OIOBSTSAMLAssertionToIDCardResponseModelBuilder;
import dk.sosi.seal.pki.SOSITestFederation;
import dk.sosi.seal.vault.ClasspathCredentialVault;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.CredentialVaultTestUtil;
import dk.sosi.seal.vault.EmptyCredentialVault;
import org.junit.Assume;
import org.junit.Before;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.Properties;

import static org.junit.Assert.*;

public class OIOBSTSAMLAssertionToIDCardResponseTest {
	private SOSIFactory sosiFactory;
	private static final OIOSAMLFactory OIOSAML_FACTORY = new OIOSAMLFactory();

	public CredentialVault idCardSigningVault;
	public CredentialVault responseSigningVault;

	@Before
	public void setup() {
		idCardSigningVault = CredentialVaultTestUtil.getMoces3CredentialVault();
		responseSigningVault = new ClasspathCredentialVault(System.getProperties(), "NSP_Test_Service_Consumer_sds.p12", "Test1234");

		sosiFactory = new SOSIFactory(idCardSigningVault, new Properties());
	}

	private UserIDCard sign(UserIDCard unsignedIdCard) {

		Request tmpRequest = sosiFactory.createNewRequest(false, "flow");
		tmpRequest.setIDCard(unsignedIdCard);
		unsignedIdCard.sign(tmpRequest.serialize2DOMDocument(), sosiFactory.getSignatureProvider());
		return unsignedIdCard;
	}
	@Test
	public void testSAMLAssertionToIDCardResponseModelBuilder() {
		// GIVEN
		OIOBSTSAMLAssertionToIDCardResponseModelBuilder modelBuilder = OIOSAML_FACTORY.createOIOBSTSAMLAssertionToIDCardResponseModelBuilder();
		Document build = createResponseDocument();

		// WHEN
		OIOBSTSAMLAssertionToIDCardResponse response = modelBuilder.build(build);

		// THEN
		response.validateSignature();
		response.validateSignatureAndTrust(new SOSITestFederation(new Properties(), "UI:DK-O:G:8d3fa047-c77e-47e4-bdd2-e91488610ce6"));
		assertEquals("Uventet action", "http://docs.oasis-open.org/ws-sx/ws-trust/200512/RST/Issue", response.getAction());
		assertEquals("Uventet Applies To/Audience Restriction", "https://fmk", response.getAppliesTo());
		assertEquals("Uventet Context uuid", "urn:uuid:b216a8d9-0cab-40f7-8f60-8fa854c284a7", response.getContext());
		assertNotNull("Forventede Created Date i response", response.getCreated());
		assertNotNull("Forventede Expireds Date i response", response.getExpires());
		assertFalse("Forventede ikke fejl", response.isFault());
		assertNull("Forventede ikke fejl info", response.getFaultActor());
		assertNull("Forventede ikke fejl info", response.getFaultCode());
		assertNull("Forventede ikke fejl info", response.getFaultString());
		assertNotNull("Forventede Message uuid i reponse", response.getMessageID());
		assertEquals("Uventet Relates To", "urn:uuid:bfe03422-990c-49ec-9a31-07eeb82ffed3", response.getRelatesTo());
		assertEquals("Uventet Signing Certificate", responseSigningVault.getSystemCredentialPair().getCertificate(), response.getSigningCertificate());
		assertEquals("Uventet Token Type", "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0", response.getTokenType());

		assertTrue("Forventede UserIDCard", response.getIDCard() instanceof UserIDCard);
		UserIDCard idCard = (UserIDCard) response.getIDCard();

		assertEquals("Uventet Alternativ Identifier/Subject Name ID", "KorsbaekKommune\\MSK", idCard.getAlternativeIdentifier());
		assertEquals("Uventet AuthenticationLevel", 4, idCard.getAuthenticationLevel().getLevel());
		assertNull("Forventede ikke cert hash i ID Card", idCard.getCertHash());
		assertNotNull("Forventede Created Date i ID Card", idCard.getCreatedDate());
		assertNotNull("Forventede Expiry Date i ID Card", idCard.getExpiryDate());
		assertNotNull("Forventede ID Card id i ID Card", idCard.getIDCardID());
		assertTrue("Uventet issuer", idCard.getIssuer().endsWith("TheSOSILibrary"));
		assertNull("Forventede ikke password i ID Card", idCard.getPassword());
		assertEquals("Uventet Signing certificate", sosiFactory.getCredentialVault().getSystemCredentialPair().getCertificate(), idCard.getSignedByCertificate());
		assertNull("Forventede ikke username i ID Card", idCard.getUsername());
		assertEquals("Uventet version", "1.0.1", idCard.getVersion());

		SystemInfo systemInfo = idCard.getSystemInfo();
		assertEquals("Uventet IT System Name", "Korsbæk Kommunes IT systemer", systemInfo.getITSystemName());
		assertEquals("Uventet Care Provider Id", "20301823", systemInfo.getCareProvider().getID());
		assertEquals("Uventet Care Provider Organization Name", "Korsbæk Kommune", systemInfo.getCareProvider().getOrgName());
		assertEquals("Uventet Care Provider Id Type", "medcom:cvrnumber", systemInfo.getCareProvider().getType());

		UserInfo userInfo2 = idCard.getUserInfo();
		assertEquals("Uventet Authorization Code", "ZXCVB", userInfo2.getAuthorizationCode());
		assertEquals("Uventet CPR", "1802602810", userInfo2.getCPR());
		assertNull("Forventede ikke Email i ID card", userInfo2.getEmail());
		assertEquals("Uventet Given Name", "Mads", userInfo2.getGivenName());
		assertNull("Forventede ikke Occupation i ID card", userInfo2.getOccupation());
		assertEquals("Uventet Role", "7170", userInfo2.getRole());
		assertEquals("Uventet Sur Name", "Skjern", userInfo2.getSurName());

		assertTrue("Forventede ID card var valid", idCard.isValidInTime());

		idCard.validateSignature();
		idCard.validateSignatureAndTrust(idCardSigningVault);
	}

	@Test
	public void testBrokenMessageSignature() throws Exception {
		//GIVEN
		OIOBSTSAMLAssertionToIDCardResponseModelBuilder modelBuilder = OIOSAML_FACTORY.createOIOBSTSAMLAssertionToIDCardResponseModelBuilder();
		Document responseDocument = createResponseDocument();
		Element issuer = getFirstChild(responseDocument.getDocumentElement(), "urn:oasis:names:tc:SAML:2.0:assertion", "Issuer");
		issuer.getFirstChild().setNodeValue("modified");

		//THEN
		OIOBSTSAMLAssertionToIDCardResponse response = modelBuilder.build(responseDocument);

		try {
			response.validateSignature();
			fail("signature validation should fail");
		} catch (Exception e) {
			assertTrue(e.getMessage(), e.getMessage().contains("signature could not be validated"));
		}
	}

	@Test
	public void testUntrustedSignature() throws Exception {
		// GIVEN
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder responseDOMBuilder = createResponseDOMBuilder(createUserIdCard());
		responseDOMBuilder.setSigningVault(idCardSigningVault);
		OIOBSTSAMLAssertionToIDCardResponseModelBuilder modelBuilder = OIOSAML_FACTORY.createOIOBSTSAMLAssertionToIDCardResponseModelBuilder();
		Document build = responseDOMBuilder.build();

		// WHEN
		OIOBSTSAMLAssertionToIDCardResponse response = modelBuilder.build(build);

		try {
			response.validateSignatureAndTrust(new SOSITestFederation(new Properties()));
			fail("Validation should fail");
		} catch (Exception e) {
			assertTrue(e.getMessage(), e.getMessage().contains("The certificate that signed the security token is not trusted!"));
		}
	}

	@Test
	public void testMissingMessageSignature() throws Exception {
		testMissingMessageSignature("body");
		testMissingMessageSignature("ts");
		testMissingMessageSignature("messageID");
		testMissingMessageSignature("action");
	}

	private void testMissingMessageSignature(String messageUri) throws Exception {
		//GIVEN
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder responseDOMBuilder = createResponseDOMBuilder(createUserIdCard());
		Document responseDocument = DOMBuilderHelper.buildWithoutSigning(responseDOMBuilder);
		new MissingMessageSignatureSigner(responseSigningVault, responseDocument).skipUri(messageUri).enhanceAndSign();
		MissingMessageSignatureValidator messageSkippingValidator =
				new MissingMessageSignatureValidator(new EmptyCredentialVault(), responseDocument.getDocumentElement()).skipUri(messageUri);
		OIOBSTSAMLAssertionToIDCardResponseModelBuilder modelBuilder = OIOSAML_FACTORY.createOIOBSTSAMLAssertionToIDCardResponseModelBuilder();

		//WHEN
		OIOBSTSAMLAssertionToIDCardResponse response = modelBuilder.build(responseDocument);

		//THEN
		messageSkippingValidator.validateSignature(); // Should not fail
		try {
			response.validateSignature();
			fail("signature validation should fail");
		} catch (Exception e) {
			assertTrue(e.getMessage(), e.getMessage().contains("Missing Liberty signature on element "));
		}
	}

	@Test
	public void testMissingAudienceRestriction() throws Exception {
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder responseDOMBuilder = createResponseDOMBuilder(createUserIdCard());
		responseDOMBuilder.setAudienceRestriction(null);
		try {
			responseDOMBuilder.build();
			fail("Build should fail");
		} catch (ModelException e) {
			assertTrue(e.getMessage(), e.getMessage().contains("audience is mandatory - but was null"));
		}
	}

	@Test
	public void testMissingSigningVault() throws Exception {
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder responseDOMBuilder = createResponseDOMBuilder(createUserIdCard());
		responseDOMBuilder.setSigningVault(null);
		try {
			responseDOMBuilder.build();
			fail("Build should fail");
		} catch (ModelException e) {
			assertTrue(e.getMessage(), e.getMessage().contains("signingVault is mandatory - but was null"));
		}
	}

	@Test
	public void testMissingContext() throws Exception {
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder responseDOMBuilder = createResponseDOMBuilder(createUserIdCard());
		responseDOMBuilder.setContext(null);
		try {
			responseDOMBuilder.build();
			fail("Build should fail");
		} catch (ModelException e) {
			assertTrue(e.getMessage(), e.getMessage().contains("context is mandatory - but was null"));
		}
	}

	@Test
	public void testMissingRelatesTo() throws Exception {
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder responseDOMBuilder = createResponseDOMBuilder(createUserIdCard());
		responseDOMBuilder.setRelatesTo(null);
		try {
			responseDOMBuilder.build();
			fail("Build should fail");
		} catch (ModelException e) {
			assertTrue(e.getMessage(), e.getMessage().contains("relatesTo is mandatory - but was null"));
		}
	}

	@Test
	public void testMissingIdCard() throws Exception {
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder responseDOMBuilder = createResponseDOMBuilder(createUserIdCard());
		responseDOMBuilder.setIDCard(null);
		try {
			responseDOMBuilder.build();
			fail("Build should fail");
		} catch (ModelException e) {
			assertTrue(e.getMessage(), e.getMessage().contains("IDCard is mandatory - but was null"));
		}
	}

	private Document createResponseDocument() {
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder domBuilder = createResponseDOMBuilder(createUserIdCard());
		return domBuilder.build();
	}

	private OIOBSTSAMLAssertionToIDCardResponseDOMBuilder createResponseDOMBuilder(UserIDCard idCard) {
		OIOBSTSAMLAssertionToIDCardResponseDOMBuilder domBuilder = OIOSAML_FACTORY.createOIOBSTSAMLAssertionToIDCardResponseDOMBuilder();
		domBuilder.setContext("urn:uuid:b216a8d9-0cab-40f7-8f60-8fa854c284a7");
		domBuilder.setRelatesTo("urn:uuid:bfe03422-990c-49ec-9a31-07eeb82ffed3");
		domBuilder.setSigningVault(responseSigningVault);
		domBuilder.setIDCard(idCard);
		domBuilder.setAudienceRestriction("https://fmk");
		return domBuilder;
	}

	private UserIDCard createUserIdCard() {
		UserInfo userInfo = new UserInfo("1802602810", "Mads", "Skjern", null, null, "7170", "ZXCVB");
		CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "20301823", "Korsbæk Kommune");
		UserIDCard userIDCard = sosiFactory.createNewUserIDCard("Korsbæk Kommunes IT systemer", userInfo, careProvider, AuthenticationLevel.MOCES_TRUSTED_USER, null, null, null, "KorsbaekKommune\\MSK");
		sign(userIDCard);
		return userIDCard;
	}

	private static Element getFirstChild(Element ancestor, String tagNS, String tagName) {
		return (Element) getChildren(ancestor, tagNS, tagName).item(0);
	}

	private static NodeList getChildren(Element ancestor, String tagNS, String tagName) {
		return tagNS != null ? ancestor.getElementsByTagNameNS(tagNS, tagName) : ancestor.getElementsByTagName(tagName);
	}
}
