package dk.sosi.seal.pki;

import static org.junit.Assert.assertEquals;

import java.security.cert.X509Certificate;
import java.util.Collections;

import javax.security.auth.x500.X500Principal;

import org.junit.Assert;
import org.junit.Test;

import dk.sosi.seal.vault.CredentialVaultTestUtil;

public class TestDistinguishedName {
	@Test
	public void testOces2ProduktionRodCertifikat() {
		/* OCES2 Certifikater - Rod certifikater - Prod */
		Assert.assertEquals("Forventede antal OCES2 certifikater i OCESCertificationAuthority", 1, OCESCertificationAuthority.OCES_2_ROOT_CERTIFICATES.size());

		X509Certificate certificate = OCESCertificationAuthority.OCES_2_ROOT_CERTIFICATE;
		X500Principal principal = certificate.getSubjectX500Principal();
		DistinguishedName actual = new DistinguishedName(principal);
		assertEquals("Forkert CommonName", "TRUST2408 OCES Primary CA", actual.getCommonName());
		assertEquals("Forkert Country", "DK", actual.getCountry());
		assertEquals("Forkert Organization", "TRUST2408", actual.getOrganization());
		assertEquals("Forkert OrganizationalUnit", null, actual.getOrganizationalUnit());
		assertEquals("Forkert OrganizationalUnits", null, actual.getOrganizationalUnits());
		assertEquals("Forkert OrganizationIdentifier", null, actual.getOrganizationIdentifier());
		assertEquals("Forkert SubjectSerialNumber", null, actual.getSubjectSerialNumber());
		assertEquals("Forkert GivenName", null, actual.getGivenName());
		assertEquals("Forkert SurName", null, actual.getSurName());
		assertEquals("Forkert Pseudonym", null, actual.getPseudonym());
	}

	@Test
	public void testOces2TestRodCertifikat() {
		/* OCES2 Certifikater - Rod certifikater - Test */
		Assert.assertEquals("Forventede antal OCES2 certifikater i OCESTestCertificationAuthority", 1, OCESTestCertificationAuthority.OCES_2_TEST_ROOT_CERTIFICATES.size());

		X509Certificate certificate = OCESTestCertificationAuthority.OCES_2_TEST_ROOT_CERTIFICATE;
		X500Principal principal = certificate.getSubjectX500Principal();
		DistinguishedName actual = new DistinguishedName(principal);
		assertEquals("Forkert CommonName", "TRUST2408 Systemtest VII Primary CA", actual.getCommonName());
		assertEquals("Forkert Country", "DK", actual.getCountry());
		assertEquals("Forkert Organization", "TRUST2408", actual.getOrganization());
		assertEquals("Forkert OrganizationalUnit", null, actual.getOrganizationalUnit());
		assertEquals("Forkert OrganizationalUnits", null, actual.getOrganizationalUnits());
		assertEquals("Forkert OrganizationIdentifier", null, actual.getOrganizationIdentifier());
		assertEquals("Forkert SubjectSerialNumber", null, actual.getSubjectSerialNumber());
		assertEquals("Forkert GivenName", null, actual.getGivenName());
		assertEquals("Forkert SurName", null, actual.getSurName());
		assertEquals("Forkert Pseudonym", null, actual.getPseudonym());
	}

	@Test
	public void testOces3ProduktionRodCertifikat() {
		/* OCES3 Certifikater - Rod certifikater - Prod */
		Assert.assertEquals("Forventede antal OCES3 certifikater i OCESCertificationAuthority", 1, OCESCertificationAuthority.OCES_3_ROOT_CERTIFICATES.size());

		X500Principal principal = OCESCertificationAuthority.OCES_3_ROOT_CERTIFICATES.get(0).getSubjectX500Principal();
		DistinguishedName actual = new DistinguishedName(principal);

		assertEquals("Forkert CommonName", "Den Danske Stat OCES rod-CA", actual.getCommonName());
		assertEquals("Forkert Country", "DK", actual.getCountry());
		assertEquals("Forkert Organization", "Den Danske Stat", actual.getOrganization());
		assertEquals("Forkert OrganizationalUnit", null, actual.getOrganizationalUnit());
		assertEquals("Forkert OrganizationalUnits", null, actual.getOrganizationalUnits());
		assertEquals("Forkert OrganizationIdentifier", null, actual.getOrganizationIdentifier());
		assertEquals("Forkert SubjectSerialNumber", null, actual.getSubjectSerialNumber());
		assertEquals("Forkert GivenName", null, actual.getGivenName());
		assertEquals("Forkert SurName", null, actual.getSurName());
		assertEquals("Forkert Pseudonym", null, actual.getPseudonym());
	}

	@Test
	public void testOces3TestRodCertifikat() {
		/* OCES3 Certifikater - Rod certifikater - Test */
		Assert.assertEquals("Forventede antal OCES3 certifikater i OCESTestCertificationAuthority", 2, OCESTestCertificationAuthority.OCES_3_TEST_ROOT_CERTIFICATES.size());

		X500Principal principal;
		DistinguishedName actual;

		principal = OCESTestCertificationAuthority.OCES_3_TEST_ROOT_CERTIFICATES.get(0).getSubjectX500Principal();
		actual = new DistinguishedName(principal);
		assertEquals("Forkert CommonName", "Den Danske Stat OCES rod-CA", actual.getCommonName());
		assertEquals("Forkert Country", "DK", actual.getCountry());
		assertEquals("Forkert Organization", "Den Danske Stat", actual.getOrganization());
		assertEquals("Forkert OrganizationalUnit", "Test - cti", actual.getOrganizationalUnit());
		assertEquals("Forkert OrganizationalUnits", Collections.singleton("Test - cti"), actual.getOrganizationalUnits());
		assertEquals("Forkert OrganizationIdentifier", null, actual.getOrganizationIdentifier());
		assertEquals("Forkert SubjectSerialNumber", null, actual.getSubjectSerialNumber());
		assertEquals("Forkert GivenName", null, actual.getGivenName());
		assertEquals("Forkert SurName", null, actual.getSurName());
		assertEquals("Forkert Pseudonym", null, actual.getPseudonym());

		principal = OCESTestCertificationAuthority.OCES_3_TEST_ROOT_CERTIFICATES.get(1).getSubjectX500Principal();
		actual = new DistinguishedName(principal);
		assertEquals("Forkert CommonName", "Den Danske Stat OCES rod-CA", actual.getCommonName());
		assertEquals("Forkert Country", "DK", actual.getCountry());
		assertEquals("Forkert Organization", "Den Danske Stat", actual.getOrganization());
		assertEquals("Forkert OrganizationalUnit", "Test - certifikat.local", actual.getOrganizationalUnit());
		assertEquals("Forkert OrganizationalUnits", Collections.singleton("Test - certifikat.local"), actual.getOrganizationalUnits());
		assertEquals("Forkert OrganizationIdentifier", null, actual.getOrganizationIdentifier());
		assertEquals("Forkert SubjectSerialNumber", null, actual.getSubjectSerialNumber());
		assertEquals("Forkert GivenName", null, actual.getGivenName());
		assertEquals("Forkert SurName", null, actual.getSurName());
		assertEquals("Forkert Pseudonym", null, actual.getPseudonym());
	}

	@Test
	public void testVoces3TestCertifikat() {
		/* VOCES3 Certifikater - Test */
		X509Certificate certificate = CredentialVaultTestUtil.getVoces3CredentialVault().getSystemCredentialPair().getCertificate();
		X500Principal principal = certificate.getSubjectX500Principal();
		DistinguishedName actual = new DistinguishedName(principal);
		assertEquals("Forkert CommonName", "VOCES_gyldig", actual.getCommonName());
		assertEquals("Forkert Country", "DK", actual.getCountry());
		assertEquals("Forkert Organization", "Testorganisation nr. 94354969", actual.getOrganization());
		assertEquals("Forkert OrganizationalUnit", null, actual.getOrganizationalUnit());
		assertEquals("Forkert OrganizationalUnits", null, actual.getOrganizationalUnits());
		assertEquals("Forkert OrganizationIdentifier", "NTRDK-94354969", actual.getOrganizationIdentifier());
		assertEquals("Forkert SubjectSerialNumber", "UI:DK-O:G:c70b0207-162e-4d3d-a7f1-a19a8e07d99b", actual.getSubjectSerialNumber());
		assertEquals("Forkert GivenName", null, actual.getGivenName());
		assertEquals("Forkert SurName", null, actual.getSurName());
		assertEquals("Forkert Pseudonym", null, actual.getPseudonym());
	}

	@Test
	public void testMoces3TestCertifikat() {
		/* MOCES3 Certifikater - Test */
		X509Certificate certificate = CredentialVaultTestUtil.getMoces3CredentialVault().getSystemCredentialPair().getCertificate();
		X500Principal principal = certificate.getSubjectX500Principal();
		DistinguishedName actual = new DistinguishedName(principal);
		assertEquals("Forkert CommonName", "Thormund Nissen", actual.getCommonName());
		assertEquals("Forkert Country", "DK", actual.getCountry());
		assertEquals("Forkert Organization", "Testorganisation nr. 91026150", actual.getOrganization());
		assertEquals("Forkert OrganizationalUnit", null, actual.getOrganizationalUnit());
		assertEquals("Forkert OrganizationalUnits", null, actual.getOrganizationalUnits());
		assertEquals("Forkert OrganizationIdentifier", "NTRDK-91026150", actual.getOrganizationIdentifier());
		assertEquals("Forkert SubjectSerialNumber", "UI:DK-E:G:dc7f4a98-d4f9-4d77-84f9-ae83c4e0bf61", actual.getSubjectSerialNumber());
		assertEquals("Forkert GivenName", "Thormund", actual.getGivenName());
		assertEquals("Forkert SurName", "Nissen", actual.getSurName());
		assertEquals("Forkert Pseudonym", null, actual.getPseudonym());
	}
}
