/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.pki;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.security.cert.X509Certificate;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;

import dk.sosi.seal.test.TestParameterBuilder;
import dk.sosi.seal.vault.CredentialVaultTestUtil;

@RunWith(Parameterized.class)
public class TestOCESUtil {
	@Parameter(0)
	public String suiteName;
	
	@Parameter(1)
	public X509Certificate moces;

	@Parameter(2)
	public X509Certificate voces;

	@Parameter(3)
	public X509Certificate foces;

	/*Setting up test parameters*/
	@Parameters(name = "{0}")
	public static Iterable<Object[]> data() {
		TestParameterBuilder builder = new TestParameterBuilder();
		builder.newTest("OCES3").add(CredentialVaultTestUtil.getMoces3CredentialVault().getSystemCredentialPair().getCertificate(),
				CredentialVaultTestUtil.getVoces3CredentialVault().getSystemCredentialPair().getCertificate(),
				CredentialVaultTestUtil.loadCertificate("oces3/FOCES_gyldig.p12"));
		return builder.build();
	}

	@Test
	public void testIsProbableOCES1Certificate() {
		assertTrue(OCESUtil.isProbableOCES1Certificate(moces) == ("OCES1".equals(suiteName)));
		assertTrue(OCESUtil.isProbableOCES1Certificate(voces) == ("OCES1".equals(suiteName)));
		assertTrue(OCESUtil.isProbableOCES1Certificate(foces) == ("OCES1".equals(suiteName)));
	}

	@Test
	public void testIsProbableOCES2Certificate() {
		assertTrue(OCESUtil.isProbableOCES2Certificate(moces) == ("OCES2".equals(suiteName)));
		assertTrue(OCESUtil.isProbableOCES2Certificate(voces) == ("OCES2".equals(suiteName)));
		assertTrue(OCESUtil.isProbableOCES2Certificate(foces) == ("OCES2".equals(suiteName)));
	}

	@Test
	public void testIsProbableOCES3Certificate() {
		assertTrue(OCESUtil.isProbableOCES3Certificate(moces) == ("OCES3".equals(suiteName)));
		assertTrue(OCESUtil.isProbableOCES3Certificate(voces) == ("OCES3".equals(suiteName)));
		assertTrue(OCESUtil.isProbableOCES3Certificate(foces) == ("OCES3".equals(suiteName)));
	}

	@Test
	public void testMocesIsIntermediateOrRootCertificate() {
		assertFalse(OCESUtil.isProbableIntermediateOrRootCertificate(moces));
		assertFalse(OCESUtil.isProbableIntermediateOrRootCertificate(voces));
		assertFalse(OCESUtil.isProbableIntermediateOrRootCertificate(foces));
	}
}
