/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.pki.internal;

import dk.sosi.seal.pki.DistinguishedName;
import org.junit.Test;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

/**
 */
public class TestSOSISTSCertificateMatcher {

    @Test
    public void testMatcher() {
        final SOSISTSCertificateMatcher matcher = new SOSISTSCertificateMatcher("SOSI Federation");
        DistinguishedName dn = new DistinguishedName("CN=Test Bruger 2 + SERIALNUMBER=CVR:25767535-RID:1118061043356, O=TDC TOTALLØSNINGER A/S // CVR:25767535, C=DK");
        assertFalse(matcher.matches(dn));

        dn = new DistinguishedName("CN=SOSI Federation FOO + SERIALNUMBER=CVR:25767535-RID:1118061043356, O=TDC TOTALLØSNINGER A/S // CVR:25767535, C=DK");
        assertFalse(matcher.matches(dn));

        dn = new DistinguishedName("CN=Test Bruger 2 + SERIALNUMBER=CVR:33257872-FID:1118061043356, O=TDC TOTALLØSNINGER A/S // CVR:25767535, C=DK");
        assertFalse(matcher.matches(dn));

        dn = new DistinguishedName("CN=SOSI Test Federation FOO + SERIALNUMBER=CVR:33257872-FID:1118061043356, O=TDC TOTALLØSNINGER A/S // CVR:25767535, C=DK");
        assertFalse(matcher.matches(dn));

        dn = new DistinguishedName("CN=SOSI Federation FOO + SERIALNUMBER=CVR:33257872-FID:1118061043356, O=TDC TOTALLØSNINGER A/S // CVR:25767535, C=DK");
        assertTrue(matcher.matches(dn));

        dn = new DistinguishedName("CN=SOSI Federation FOO + SERIALNUMBER=CVR:55832218-FID:1118061043356, O=TDC TOTALLØSNINGER A/S // CVR:25767535, C=DK");
        assertFalse(matcher.matches(dn));

        // Old STS certificate
        dn = new DistinguishedName("cn=Danske Regioner - SOSI STS+serialNumber=CVR:55832218-UID:1165408969529,o=Danske Regioner // CVR:55832218,c=DK");
        assertFalse(matcher.matches(dn));

    }
}
