package dk.sosi.seal.sign;

import java.util.Date;
import java.util.Properties;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.*;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import dk.sosi.seal.pki.CertificateInfo;
import dk.sosi.seal.vault.ClasspathCredentialVault;
import dk.sosi.seal.vault.CredentialVault;

public class BootstrapTokenHelper {

	private CredentialVault idpVault;

	public BootstrapTokenHelper() {
		this(new ClasspathCredentialVault(new Properties(), "TEST trusted IdP SOSI alias (til bootstrap token).jks", "Test1234"));
	}

	public BootstrapTokenHelper(CredentialVault idpVault) {
		this.idpVault = idpVault;
	}

	public OIOSAMLAssertionBuilder createOIOSAMLAssertionBuilder(String pid) {
		OIOSAMLAssertionBuilder assertionBuilder = new OIOSAMLAssertionBuilder() {

			@Override
			protected Element createAttributeStatement(Document doc) {
				return null;
			}

			@Override
			protected Node createAuthnStatement() {
				return null;
			}

			@Override
			public void validateBeforeBuild() {

				this.certificateInfo = CertificateInfo.fromString("SubjectDN={C=DK,O=Ingen organisatorisk tilknytning,CN=Lars Larsen,Serial="+pid+"},IssuerDN={C=ignored},CertSerial={1234}");
			}

		};
		assertionBuilder.setSigningVault(idpVault);
		assertionBuilder.setIssuer("TEST trusted IdP");
		assertionBuilder.setRecipientURL("https://sosi");
		assertionBuilder.setAudienceRestriction("https://bootstrap.sts.nspop.dk/");
		Date now = new Date();
		assertionBuilder.setDeliveryNotOnOrAfter(new Date(now.getTime() + 5 * 60 * 1000));
		assertionBuilder.setNotBefore(new Date(now.getTime() - 5 * 1000));
		assertionBuilder.setNotOnOrAfter(new Date(now.getTime() + 5 * 60 * 1000));
		return assertionBuilder;
	}


	public OIOSAMLAssertion buildBootstrapAssertion(String pid) {
		OIOSAMLAssertionBuilder builder = createOIOSAMLAssertionBuilder(pid);
		builder.setUserIdCard(createUserIDCard());
		return builder.build();
	}

	private UserIDCard createUserIDCard() {
		SOSIFactory sosiFactory = new SOSIFactory(idpVault, System.getProperties());
		CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "30808460", "Lægehuset på bakken");
		UserInfo userInfo = new UserInfo("1111111118", "Hans", "Dampf", null, null, "7170", "341KY");
		String alternativeIdentifier = new CertificateInfo(idpVault.getSystemCredentialPair().getCertificate()).toString();
		UserIDCard userIDCard = sosiFactory.createNewUserIDCard("IT-System", userInfo, careProvider, AuthenticationLevel.MOCES_TRUSTED_USER, null, null, null, alternativeIdentifier);
		Request tmpRequest = sosiFactory.createNewRequest(false, "flow");
		tmpRequest.setIDCard(userIDCard);
		userIDCard.sign(tmpRequest.serialize2DOMDocument(), sosiFactory.getSignatureProvider());
		return userIDCard;
	}
}
