/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Klasses TestParameter vil findes hver parametriseret test.
 * Klassen vil indeholde 1 navn, 1 liste af valideringer og et antal parametre.
 * 
 * Testklasser der bruges TestParameterBuilder skal have følgende opbygning (navnene på alle parametere er kun eksempler):
 * 
 *<pre>
 *@Parameter(0)
 *public String name;
 *@Parameter(1)
 *public Object parameter1;
 *...
 *@Parameter(n)
 *public Object parameterN;
 *</pre>
 */
public class TestParameter {
	/**
	 * Navnet på denne parametriserede test
	 */
	private final String name;
	/**
	 * De parametre der indfår i testen
	 */
	private List<Object> parameters = new ArrayList<Object>();

	/**
	 *  Ny Parametriseret test.
	 *  
	 * @param name Navnet på den parametriserede test
	 */
	public TestParameter(String name) {
		this.name = name;
	}

	/*Info*/
	/**
	 * Metoden bruges af TestParameterBuilder til at opbygge dobbelt array med testdata.
	 * @return
	 */
	public int size() {
		return parameters.size() + 1; //Name
	}

	/*Input*/
	/**
	 * Tilføjer et antal parametre til testen
	 * 
	 * @param objs
	 * @return
	 */
	public TestParameter add(Object... objs) {
		parameters.addAll(Arrays.asList(objs));
		return this;
	}

	/*Getters*/
	public String getName() {
		return name;
	}

	public List<Object> getParameters() {
		return Collections.unmodifiableList(parameters);
	}
}
