/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.seal.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Klasse til at oprette de parametriserede tests og tilhørende data.
 * Resultatet af build er det som frameworket for parametriserede tests forventer.
 * 
 */
public class TestParameterBuilder {
	/**
	 * Listen af parametriserede tests
	 */
	private final List<TestParameter> testDataList = new ArrayList<>();
	/**
	 * Holder til aktuelle parametriserede test
	 */
	private TestParameter current = null;
	/**
	 * Opret en ny parametriseret test.
	 * 
	 * @param name Navnet på den parametriserede test
	 * @return Nyt TestParameter Object som indeholder alle parametre til den parametriserede test.
	 */
	public TestParameter newTest(String name) {
		current = new TestParameter(name);
		testDataList.add(current);
		return current;
	}
	/**
	 * Framework for parametrisret test forventer et dobbelt array af Objekter. Her er det så en liste af Objekt-arrays.
	 * Hvert række i listen er en parametrisret test og Objekt-array for rækken er parametrene til testen.
	 * 
	 * @return En liste arrays indeholdene parametre (Object)
	 */
	public Iterable<Object[]> build() {
		// Hvor mange tests er der (testDataList.size()) og hvor mange parametre er der (testDataList.get(0).size() eller 0)
		Object[][] result = new Object[testDataList.size()][testDataList.size() > 0 ? testDataList.get(0).size() : 0];
		// For hver test
		for (int i = 0; i < testDataList.size(); i++) {
			int index = 0;
			TestParameter testData = testDataList.get(i);
			//Tilføjes navn på testen i index=0
			result[i][index++] = testData.getName();
			//Parametrene i index=1,.....,index=n
			List<Object> parameters = testData.getParameters();

			for (Object obj : parameters) {
				result[i][index++] = obj;
			}
		}
		// Returnere listen af arrays
		return Arrays.asList(result);
	}
}
