/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.sosi.seal.vault;

import junit.framework.TestCase;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.Properties;

import static dk.sosi.seal.vault.GenericCredentialVault.KeyStoreType;

public class TestCredentialVaultTrust extends TestCase {

	private static final String KEYSTORE_PATH = "oces3/MOCES_gyldig.p12";
	private static final String KEYSTORE_PASSPHRASE = "Test1234";
	private static final String TRUSTED_ALIAS = "1";

	public void testClasspathCredentialVault() throws Exception {
		GenericCredentialVault target = new ClasspathCredentialVault(null, KEYSTORE_PATH, KEYSTORE_PASSPHRASE);

		X509Certificate trustedCert = target.getTrustedCertificate(TRUSTED_ALIAS);
		assertNotNull("no trusted cert alias with name " + TRUSTED_ALIAS, trustedCert);

		assertTrue("alias " + TRUSTED_ALIAS + " not trusted", target.isTrustedCertificate(trustedCert));
	}

	public void testFileBasedCredentialVault() throws CredentialVaultException, IOException {
		GenericCredentialVault target = new FileBasedCredentialVault(null, CredentialVaultTestUtil.saveResourceToTempFile("/" + KEYSTORE_PATH), KEYSTORE_PASSPHRASE);

		X509Certificate trustedCert = target.getTrustedCertificate(TRUSTED_ALIAS);
		assertNotNull("no trusted cert alias with name " + TRUSTED_ALIAS, trustedCert);

		assertTrue("alias " + TRUSTED_ALIAS + " not trusted", target.isTrustedCertificate(trustedCert));
	}

	// TODO SDS-6659
	public void _testGenericCredentialVault() throws IOException, GeneralSecurityException {
		GenericCredentialVault target = newGenericCredentialVault();

		X509Certificate trustedCert = target.getTrustedCertificate(TRUSTED_ALIAS);
		assertNotNull("no trusted cert alias with name " + TRUSTED_ALIAS, trustedCert);

		assertTrue("alias " + TRUSTED_ALIAS + " not trusted", target.isTrustedCertificate(trustedCert));

		X509Certificate systemCert =  target.getSystemCredentialPair().getCertificate();
		assertNotNull("no system certificate", systemCert);

		assertTrue("system certificate not trusted", target.isTrustedCertificate(systemCert));
	}

	private GenericCredentialVault newGenericCredentialVault() throws IOException, GeneralSecurityException {
		KeyStore keyStore = KeyStore.getInstance(KeyStoreType.Pkcs12.toString());
		try (BufferedInputStream bis = new BufferedInputStream(TestCredentialVaultTrust.class.getClassLoader()
																	   .getResourceAsStream(KEYSTORE_PATH))) {
			keyStore.load(bis, KEYSTORE_PASSPHRASE.toCharArray());
		}
		return new GenericCredentialVault(new Properties(), keyStore, KEYSTORE_PASSPHRASE);
	}

}
