/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sosi.seal.xml;

import dk.sosi.seal.model.ModelException;
import org.junit.Before;
import org.junit.Test;
import org.xml.sax.InputSource;

import static org.junit.Assert.*;


public class ClasspathResourceResolverTest {
    ClasspathResourceResolver subject;

    @Before
    public void setup() {
        subject = new ClasspathResourceResolver();
    }

    @Test
    public void resolveEntity_success() throws Exception {
        // Arrange
        String publicId = "foo";
        String systemId = "/medcom.xsd";

        // Act
        InputSource is = subject.resolveEntity(publicId, systemId);

        // Assert
        assertEquals(publicId, is.getPublicId());
        assertEquals(systemId, is.getSystemId());
    }

    @Test
    public void resolveEntity_noSlash_success() throws Exception {
        // Arrange
        String publicId = "foo";
        String systemId = "medcom.xsd";

        // Act
        InputSource is = subject.resolveEntity(publicId, systemId);

        // Assert
        assertEquals(publicId, is.getPublicId());
        assertEquals(systemId, is.getSystemId());
    }

    @Test(expected = ModelException.class)
    public void resolveEntity_failure() throws Exception {
        // Arrange
        String publicId = "foo";
        String systemId = "/foo.xsd";

        // Act
        InputSource is = subject.resolveEntity(publicId, systemId);

        // Assert
    }
}