/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;

public class EndpointClient<T> {

    interface Executor<S, T> {
        Class<T> getServiceInterface();		
        S execute(T t, String input);		
    }

    private final URL endpointUrl;
    private final URL wsdlDocumentLocation;
    private final QName serviceQN;
    private final QName portQN;

    public EndpointClient(URL endpointUrl, QName serviceQN, QName portQN) throws MalformedURLException {
        this.endpointUrl = endpointUrl;
        this.wsdlDocumentLocation = new URL(endpointUrl.toString()+"?wsdl");
        this.serviceQN = serviceQN;
        this.portQN = portQN;

    }

    public <S> S execute(String input, Executor<S, T> executor) {
        Service service = Service.create(wsdlDocumentLocation, serviceQN);
        T port = service.getPort(executor.getServiceInterface());
        return executor.execute(port, input);
    }

    public URL getWsdlDocumentLocation() {
        return wsdlDocumentLocation;
    }

    public URL getEndpointAddress() {
        return endpointUrl;
    }

    public String getNamespaceURI() {
        return serviceQN != null ? serviceQN.getNamespaceURI() : portQN.getNamespaceURI();
    }

    public String getServicePart() {
        return serviceQN != null ? serviceQN.getLocalPart() : null;
    }

    public String getPortName() {
        return portQN != null ? portQN.getLocalPart() : null;
    }

    public QName getServicePartQName() {
        return serviceQN;
    }

    public QName getPortNameQName() {
        return portQN;
    }
}
