/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;

public class EndpointClientBuilder<T> {

    private final URL endpointUrl;
    private String namespaceURI = null;
    private String servicePart = null;
    private String portName = null;

    EndpointClientBuilder(URL endpointUrl) {
        this.endpointUrl = endpointUrl;
    }

    public EndpointClientBuilder<T> setNamespaceURI(String namespaceURI) {
        this.namespaceURI = namespaceURI;
        return this;
    }

    public EndpointClientBuilder<T> setServicePart(String servicePart) {
        this.servicePart = servicePart;
        return this;
    }

    public EndpointClientBuilder<T> setPortName(String portName) {
        this.portName = portName;
        return this;
    }

    public EndpointClient<T> build() throws MalformedURLException {		
        QName serviceQN = new QName(namespaceURI, servicePart);
        QName portQN = new QName(namespaceURI, portName);
        return new EndpointClient<T>(endpointUrl, serviceQN, portQN);
    }

}
