/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.ws.Binding;
import javax.xml.ws.Endpoint;

public class EndpointServer<T> implements AutoCloseable{
	private final URL endpointUrl;
	private Endpoint endpoint;
	private EndpointServer(URL endpointUrl, Endpoint ep) {
		this.endpointUrl = endpointUrl;
		this.endpoint = ep;
	}

	public static <T> EndpointServer<T> start(String endpointAddress, T serviceImpl) throws MalformedURLException {
		return start(new URL(endpointAddress), serviceImpl);
	}
	
	public static <T> EndpointServer<T> start(URL endpointUrl, T serviceImpl) {
		Endpoint ep = Endpoint.publish(endpointUrl.toString(), serviceImpl);		
		return new EndpointServer<T>(endpointUrl, ep);
	}
	
	@Override
	public void close() {
		endpoint.stop();
		endpoint = null;
	}

	public boolean isPublished() {
		return endpoint.isPublished();
	}

	public Binding getBinding() {
		return endpoint.getBinding();
	}
	
	public EndpointClientBuilder<T> getClientBuilder() {
		return new EndpointClientBuilder<T>(endpointUrl);
	}

}
