/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import dk.sosi.seal.model.SignatureConfiguration;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.model.constants.IDValues;
import dk.sosi.seal.pki.SignatureProviderFactory;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.xml.DebugErrorHandler;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class MessageGeneratorHelper {
	public static final SimpleDateFormat sdf = new SimpleDateFormat(
			"yyyy-MM-dd'T'HH:mm:ss");

	/**
	 * Parses xml file from classpath
	 * 
	 * @param filename
	 * @return
	 */
	public static Document parseXmlFile(String filename) {
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			DocumentBuilder db;
			db = dbf.newDocumentBuilder();
			db.setErrorHandler(new DebugErrorHandler(true));
			InputStream inputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(filename);
			InputSource is = new InputSource(inputStream);
			return db.parse(is);
		} catch (ParserConfigurationException e) {
			throw new MessageGeneratorException("parse xml file failed", e);
		} catch (SAXException e) {
			throw new MessageGeneratorException("parse xml file failed", e);
		} catch (IOException e) {
			throw new MessageGeneratorException("parse xml file failed", e);
		}
	}

	public static Document getAdjustedDocument(String filename) {
		try {
			Document document = parseXmlFile(filename);
			Date d = new Date();
			String dateStringNow = sdf.format(d);
			Calendar c = Calendar.getInstance();
			c.add(Calendar.DAY_OF_MONTH, 1);
			String dateStringLater = sdf.format(c.getTime());

			XPathAPI.selectSingleNode(document, "//wsu:Created")
					.getFirstChild().setNodeValue(dateStringNow);
			Element elm = (Element) XPathAPI.selectSingleNode(document,
					"//saml:Assertion");
			elm.setAttributeNS(null, "IssueInstant", dateStringNow);
			elm = (Element) XPathAPI.selectSingleNode(document,
					"//saml:Conditions");
			elm.setAttributeNS(null, "NotBefore", dateStringNow);
			elm.setAttributeNS(null, "NotOnOrAfter", dateStringLater);
			return document;
		} catch (DOMException e) {
			throw new MessageGeneratorException("adjust doc failed", e);
		} catch (TransformerException e) {
			throw new MessageGeneratorException("adjust doc failed", e);
		}
	}

	public static void writeXmlFile(Document doc, String filename) {
		String xml = XmlUtil.node2String(doc,true,true);
		PrintWriter pw = null; 
		try {
			pw = new PrintWriter(new BufferedWriter(new FileWriter(new File(filename))));
			pw.print(xml);
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			pw.close();
		}
	}


	public static Document sign(Document doc, CredentialVault vault) {
        final SignatureConfiguration configuration = new SignatureConfiguration(new String[]{IDValues.IDCARD}, IDValues.IDCARD, IDValues.id);
        SignatureUtil.sign(SignatureProviderFactory.fromCredentialVault(vault), doc, configuration);
		return doc;
	}

	public static void copyIdCard(Document from, Document to) {
		try {
			Node newIdCardNode = XPathAPI.selectSingleNode(from,
					"//saml:Assertion");
			Node oldIdCardNode = XPathAPI.selectSingleNode(to,
					"//saml:Assertion");
			oldIdCardNode.getParentNode().replaceChild(
					to.importNode(newIdCardNode, true), oldIdCardNode);
		} catch (TransformerException e) {
			throw new MessageGeneratorException("copy idcard failed", e);
		}
	}
	
	public static void removeSignature(Document providerRequest) {
		try {
			Node role = XPathAPI.selectSingleNode(providerRequest,
					"//ds:Signature[@id='OCESSignature']");
			role.getParentNode().removeChild(role);
		} catch (TransformerException e) {
			throw new MessageGeneratorException("element deletion failed",e);
		}
	}
}
