/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class SoapClient {
	private final URL url;

	private HttpURLConnection httpConn;
    private String soapAction = "\"\"";

	private final static Logger logger = Logger.getLogger(SoapClient.class);

	public SoapClient(URL SOAPUrl) {
		url = SOAPUrl;
	}

    public void setSOAPAction(String action) {
        this.soapAction = action;
    }

	public Document execute(String soapEnvelope) {
		logger.debug("SOAPClient.execute:" + soapEnvelope);
		

		try {
			httpConn = (HttpURLConnection) url.openConnection();

			final byte[] soapEnvelopeBytes = soapEnvelope.getBytes("UTF-8");
			
			httpConn.setRequestProperty("Content-Length", String.valueOf(soapEnvelopeBytes.length));
			httpConn.setRequestProperty("Content-Type", "text/xml; charset=utf-8");
			httpConn.setRequestProperty("SOAPAction", soapAction);
			httpConn.setRequestMethod("POST");
			httpConn.setDoOutput(true);
			httpConn.setDoInput(true);

			OutputStream out = httpConn.getOutputStream();
			out.write(soapEnvelopeBytes);
			out.close();

			int responseCode = httpConn.getResponseCode();
			
			if(responseCode > 200) {
				System.out.println("Svar fra "+url+"\nCode:"+responseCode+"\nMessage:"+httpConn.getResponseMessage());
			}
			
			org.xml.sax.InputSource inputSource = new InputSource(responseCode >= HttpURLConnection.HTTP_INTERNAL_ERROR ? httpConn.getErrorStream() : httpConn.getInputStream());

			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(false);
			factory.setNamespaceAware(true);
			Document document;
			try {
				DocumentBuilder builder = factory.newDocumentBuilder();
				document = builder.parse(inputSource);
			} catch (SAXException sxe) {
				Exception x = sxe;
				if (sxe.getException() != null) {
					x = sxe.getException();
				}
				throw new MessageGeneratorException(x.getMessage(),x);
			} catch (ParserConfigurationException pce) {
				throw new MessageGeneratorException(pce.getMessage(),pce);
			}
			return document;
		} catch (IOException e) {
			throw new MessageGeneratorException("Error during soap invocation: " + e.getMessage(), e);
		}
	}
}