/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import javax.jws.WebService;

import dk.sosi.seal.model.BlurringInstructions10Impl;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import dk.sosi.seal.OIOIDWSFactory;
import dk.sosi.seal.model.BasicPrivileges;
import dk.sosi.seal.model.CitizenIdentityToken;
import dk.sosi.seal.model.LibertyRequest;
import dk.sosi.seal.model.LibertyResponseDOMEnhancer;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.model.constants.SOAPTags;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.pki.SOSITestFederation;
import dk.sosi.seal.vault.ClasspathCredentialVault;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.CredentialVaultTestUtil;
import dk.sosi.seal.xml.TagUtil;
import dk.sosi.seal.xml.XmlUtil;

@WebService(endpointInterface = "dk.sosi.testtools.testrequest.DemoService", targetNamespace = "http://www.dkma.dk/medicinecard/xml.schema/2015/01/01/E1", serviceName = "GetMedicineCardRequest", portName = "GetMedicineCardPort")
public class DemoServiceImpl implements DemoService {
	@Override
	public String GetMedicineCard(String input) {		
		// Init SOSI TEST Federation
		final Federation federation = new SOSITestFederation(SignatureUtil.setupCryptoProviderForJVM());
		final CredentialVault spVault = new ClasspathCredentialVault(System.getProperties(), "TEST whitelisted SP SOSI alias.jks", "Test1234");
		final CredentialVault fmkIdwsTestVault = CredentialVaultTestUtil.getCredentialVaultFromResource(System.getProperties(), "fmk_idws_test.p12");

		
		final OIOIDWSFactory factory = new OIOIDWSFactory(federation, fmkIdwsTestVault);
		final LibertyRequest request = factory.createRequest(XmlUtil.readXml(System.getProperties(), input, false));
		
		/* Work on input... */
		ToStringBuilder json = new ToStringBuilder(this, ToStringStyle.JSON_STYLE);
		json.append("MessageID", request.getMessageID());
		json.append("CreatedTimestamp", request.getCreatedTimestamp());
		json.append("Action", request.getAction());
		json.append("To", request.getTo());
		json.append("RelatesTo", request.getRelatesTo());
		
		CitizenIdentityToken identityToken = request.getIdentityToken();
		ToStringBuilder idJson = new ToStringBuilder(this, ToStringStyle.JSON_STYLE);
		idJson.append("AssuranceLevel",identityToken.getAssuranceLevel());
		idJson.append("AudienceRestriction",identityToken.getAudienceRestriction());
		idJson.append("Cpr",identityToken.getCpr());
		idJson.append("ID",identityToken.getID());
		idJson.append("Issuer",identityToken.getIssuer());
		idJson.append("NotBefore",identityToken.getNotBefore());
		idJson.append("NotOnOrAfter",identityToken.getNotOnOrAfter());
		idJson.append("SpecVersion",identityToken.getSpecVersion());
		
		BasicPrivileges privileges = identityToken.getPrivileges();
		idJson.append("Privileges", privileges != null ? ToStringBuilder.reflectionToString(privileges, ToStringStyle.JSON_STYLE) : null);

		BlurringInstructions10Impl blurringInstructions10 = (BlurringInstructions10Impl) identityToken.getBlurringInstructions();
		idJson.append("BlurredInstructions", blurringInstructions10 != null ? ToStringBuilder.reflectionToString(blurringInstructions10, ToStringStyle.JSON_STYLE) : null);

		json.append("IdentityToken", idJson);

		/*Create simple output document*/
		Document outputDocument = XmlUtil.createEmptySoapDocument();
		Element data = TagUtil.addTag(outputDocument.getDocumentElement(), SOAPTags.envelope, SOAPTags.body, TagUtil.create("data"));
		TagUtil.setText(data, json.toString());
		/*Create simple output document*/
				
		/*Sign repsonse with test vault*/
		LibertyResponseDOMEnhancer enhancer = factory.createResponseDomEnhancer(outputDocument, true);
		
		enhancer.setWSAddressingAction("action");
		enhancer.setWSAddressingMessageID("msgId");
		enhancer.setWSAddressingRelatesTo(request.getTo());
		enhancer.setWSAddressingTo("demo client");
		
		enhancer.enhanceAndSign();

		return XmlUtil.node2String(outputDocument);
	}
}