/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.xml.XmlUtil;
import junit.framework.TestCase;
import org.w3c.dom.Document;

import java.io.File;
import java.util.Properties;

import static org.junit.Assert.assertTrue;


public class TestInteroperability // TODO SDS-6659: extends TestCase
{

	Document requestTemplate;
	ProviderMessageGenerator pmg;
	SOSIFactory factory;
	
	protected void setUp() {
		if (pmg == null) {
            Properties properties = SignatureUtil.setupCryptoProviderForJVM();
			properties.putAll(System.getProperties());
            properties.put(SOSIFactory.PROPERTYNAME_SOSI_VALIDATE_ENHANCED, "false");
            properties.put(SOSIFactory.PROPERTYNAME_SOSI_SEAL_MESSAGE_VERSION, "1.0_1");
			pmg = new ProviderMessageGenerator(properties);
			STSMessageGenerator smg = pmg.getSTSMessageGenerator();
			new File(smg.OUTPUTFOLDER + "/requests/").mkdirs();
			new File(smg.OUTPUTFOLDER + "/responses/").mkdirs();
			new File(smg.OUTPUTFOLDER + "/providerrequests/").mkdirs();

			String filename = properties.getProperty("sositest:providerrequesttemplate", "requests/lvl4providerrequest.xml");
			requestTemplate = MessageGeneratorHelper.parseXmlFile(filename);
			factory = CredentialVaultUtil.createSOSIFactoryWithTestSTSFederation(properties);
		}
	}

	// TODO SDS-6659
	public void _testDotNetLvl1SystemValidIdCard1() {
        Document doc = MessageGeneratorHelper.parseXmlFile("requests/dotnetlvl1providerrequest.xml");
		Request req = factory.deserializeRequest(XmlUtil.node2String(doc, false, true));
        String xml = XmlUtil.node2String(req.serialize2DOMDocument());
        // check - dont remove other headers 
        assertTrue(xml.indexOf("<wsa:Action/>") != -1);
        // check - dont remove body 
        assertTrue(xml.indexOf("<subscribe xmlns=\"\">false</subscribe>") != -1);
	}

	// TODO SDS-6659
	public void _testDotNetLvl1SystemValidIdCard2() {
        Document doc = MessageGeneratorHelper.parseXmlFile("requests/dotnetlvl1providerrequest2.xml");
		Request req = factory.deserializeRequest(XmlUtil.node2String(doc, false, true));
        String xml = XmlUtil.node2String(req.serialize2DOMDocument());
        // check - dont remove body 
        assertTrue(xml.indexOf("impl:civilRegistrationNumber") != -1);
	}

	// TODO SDS-6659
	public void _testLvl1CprServiceResponse() {
        Document doc = MessageGeneratorHelper.parseXmlFile("responses/cprserviceresponse.xml");
        Reply reply = factory.deserializeReply(XmlUtil.node2String(doc, false, true));
        XmlUtil.node2String(reply.serialize2DOMDocument());
	}
}