/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.sosi.testtools.testrequest;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.Message;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.test.TestParameterBuilder;
import dk.sosi.seal.xml.XmlUtil;
import dk.sosi.testtools.server.StsFaultActorValues;
import dk.sosi.testtools.server.StsFaultCodeValues;
import dk.sosi.testtools.testrequest.STSMessageGenerator.VERSION;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;
import org.w3c.dom.Document;

import java.io.File;
import java.util.Properties;

import static org.junit.Assert.*;

@RunWith(Parameterized.class)
public class TestSTSMessageGenerator {
	
	@Parameter(0)
	public String suiteName;
	
	@Parameter(1)
	public STSMessageGenerator.VERSION stsMessageVersion;

	static Document dom1system;
	static Document dom3system;
	static Document dom3user;
	static Document dom4user;
	// invalid idcards
	static Document dom3unknown;
	static Document dom4system;
	
	static STSMessageGenerator smg;
	private static SOSIFactory factory;
	private final String voces3Cvr = "94354969"; // Cvr from oces3/VOCES_gyldig.p12

	/*Setting up test parameters*/
	@Parameters(name = "{0}")
	public static Iterable<Object[]> data() {
		TestParameterBuilder builder = new TestParameterBuilder();
		
		//Kan afvikles lokalt - men ikke på Jenkins
		builder.newTest("OCES3").add(STSMessageGenerator.VERSION.OCES3);
		
		return builder.build();
	}

	@Before
	public void setUp() {
		setUp(null);
	}

	private void setUp(String cvr) {
		if(smg != null && (smg.getVersion() != stsMessageVersion || cvr != null)) {
			dom1system = null;
		}
		if (dom1system == null) {
			System.out.print(" - create dom1system");
            Properties properties = SignatureUtil.setupCryptoProviderForJVM();
			properties.putAll(System.getProperties());
			smg = new STSMessageGenerator(properties, stsMessageVersion);

			Document document = cvr != null ? smg.createSTSRequestTemplateDOM(cvr) :
					smg.createSTSRequestTemplateDOM(stsMessageVersion);

			File file = new File(smg.OUTPUTFOLDER + "/requests/");
			file.mkdirs();
			file = new File(smg.OUTPUTFOLDER + "/responses/");
			file.mkdirs();

			dom1system = smg.setLevelType(document, 1, "system");

			dom3system = smg.setLevelType(document, 3, "system");
			dom3user = smg.setLevelType(document, 3, "user");
			dom3unknown = smg.setLevelType(document, 3, "ukendt");

			dom4system = smg.setLevelType(document, 4, "system");
			dom4user = smg.setLevelType(document, 4, "user");
			factory = CredentialVaultUtil.createSOSIFactoryWithTestSTSFederation(properties);
		}
	}


// 20-2-2015 Temporary disabled unit-test until issue ASCP00059108 is resolved

// T1
//	public void testInvalidXml() throws  Exception {
//		Document domClone = smg.domClone(dom3system);
//		Document generateInvalidXmlRequest = smg.generateInvalidXmlRequest(domClone);
//
//		Source source = new DOMSource(generateInvalidXmlRequest);
//		File file = new File(smg.OUTPUTFOLDER + "/requests/" + "invalidXml" + "Request.xml");
//
//		StringWriter sw = new StringWriter();
//		Result result = new StreamResult(sw);
//		Transformer xformer = TransformerFactory.newInstance().newTransformer();
//		xformer.transform(source, result);
//		String str = sw.toString().replaceFirst("saml:Attribute" ,"smal:Attribute");
//		FileWriter fw = new FileWriter(file);
//		fw.write(str);
//		fw.close();
//
//		Document response = smg.execute(str);
//		checkDeserializeResponse(response, false); // If the XML is totally screwed up, the STS cannot return a messageID
//
//		MessageGeneratorHelper.writeXmlFile(response, smg.OUTPUTFOLDER + "/responses/" + "invalidXml" + "Response.xml");
//
//		WSTError error = smg.getError(response);
//		assertNotNull(error);
//		assertEquals(StsFaultCodeValues.WST_INVALID_REQUEST, error.getFaultCode());
//		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
//	}

//	 T2	
	@Test
	public void testInvalidVersion() {
		Document domClone = smg.domClone(dom3system);
		Document generateInvalidVersion = smg.generateInvalidVersion(domClone);
		Document response = smg.process(generateInvalidVersion, "invalidVersion");
		checkDeserializeResponse(response, false);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_REQUEST_FAILED, error.getFaultCode());
		assertEquals(StsFaultActorValues.SEAL, error.getFaultActor());
	}

	@Test
	public void testInvalidIdCardTypeLevelCombination() {
		Document domClone = smg.domClone(dom4system);
		Document generateInvalidIdCardTypeLevelCombination = smg.generateInvalidIdCardTypeLevelCombination(domClone);
		Document response = smg.process(generateInvalidIdCardTypeLevelCombination, "invalidIdCardTypeLevelCombination");		
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_BAD_REQUEST, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	public void testInvalidGyldighed() {
		Document domClone = smg.domClone(dom3system);
		Document generateInvalidGyldighed = smg.generateInvalidGyldighed(domClone);
		Document response = smg.process(generateInvalidGyldighed, "invalidGyldighed");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_INVALID_TIME_RANGE, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	public void testInvalidAutentifikationLevel() {
		Document domClone = smg.domClone(dom1system);
		Document generateInvalidAutentifikationLevel = smg.generateInvalidAutentifikationLevel(domClone);
		Document response = smg.process(generateInvalidAutentifikationLevel, "invalidAutentifikationLevel");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_BAD_REQUEST, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	public void testInvalidIdCardType() {
		Document domClone = smg.domClone(dom3unknown);
		Document generateInvalidIdCardType = smg.generateInvalidIdCardType(domClone);
		Document response = smg.process(generateInvalidIdCardType, "invalidIdCardType");
		checkDeserializeResponse(response, false);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_REQUEST_FAILED, error.getFaultCode());
		assertEquals(StsFaultActorValues.SEAL, error.getFaultActor());
	}
	
// T3	

	@Test
	public void testUnsignedLevel3() {
		Document domClone = smg.domClone(dom3system);
		Document response = smg.process(domClone,  "unsignedLevel3");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_AUTHENTICATION_BAD_ELEMENTS, error.getFaultCode());
		assertEquals(StsFaultActorValues.SEAL, error.getFaultActor());
	}

	@Test
	public void testVocesLevel4() {
		Document domClone = smg.domClone(dom4user);
		Document generateVocesLevel4 = smg.generateVocesLevel4(domClone);
		Document response = smg.process(generateVocesLevel4, "vocesLevel4");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	public void testVocesRevoked() {
		Document domClone = smg.domClone(dom3system);
		Document generateVocesRevoked = smg.generateVocesRevoked(domClone);
		Document response = smg.process(generateVocesRevoked, "vocesRevoked");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	@Ignore("SDS-6659 We have no oces3 certificate with invalid issuer")
	public void testVocesInvalidIssuer() {
		Document domClone = smg.domClone(dom3system);
		Document generateVocesInvalidIssuer = smg.generateVocesInvalidIssuer(domClone);
		Document response = smg.process(generateVocesInvalidIssuer, "vocesInvalidIssuer");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	@Ignore("SDS-6659 We have no expired oces3 certificates yet")
	public void testVocesExpired() {
		Document domClone = smg.domClone(dom3system);
		Document generateVocesExpired = smg.generateVocesExpired(domClone);
		Document response = smg.process(generateVocesExpired, "vocesExpired");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	public void testVocesInvalidSignature() {
		Document domClone = smg.domClone(dom3system);
		Document generateVocesInvalidSignature = smg.generateVocesInvalidSignature(domClone);
		Document response = smg.process(generateVocesInvalidSignature, "vocesInvalidSignature");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_AUTHENTICATION_BAD_ELEMENTS, error.getFaultCode());
		assertEquals(StsFaultActorValues.SEAL, error.getFaultActor());
	}
	

// T4
	@Test
	public void testUnsignedLevel4() {
		Document domClone = smg.domClone(dom4user);
		Document response = smg.process(domClone, "unsignedLevel4");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_AUTHENTICATION_BAD_ELEMENTS, error.getFaultCode());
		assertEquals(StsFaultActorValues.SEAL, error.getFaultActor());
	}

	@Test
	public void testMocesLevel3() {
		Document domClone = smg.domClone(dom3system);
		Document generateMocesLevel3 = smg.generateMocesLevel3(domClone);
		Document response = smg.process(generateMocesLevel3, "mocesLevel3");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());

	}

	/* Blacklisting functionality removed from STS
	@Test
    public void testMocesBlacklisted() {
		Document domClone = smg.domClone(dom4user);
		Document generateMocesBlacklisted = smg.generateMocesBlacklisted(domClone);
		Document response = smg.process(generateMocesBlacklisted, "mocesBlacklisted");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());

	}
	*/

	@Test
	@Ignore("SDS-6659 Unignore when CRA can handle oces3 certificates on test1 environment")
	public void testMocesRevoked() {
		Document domClone = smg.domClone(dom4user);
		Document generateMocesRevoked = smg.generateMocesRevoked(domClone);
		Document response = smg.process(generateMocesRevoked, "mocesRevoked");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	@Ignore("SDS-6659 We have no oces3 certificate with invalid issuer")
	public void testMocesInvalidIssuer() {
		Document domClone = smg.domClone(dom4user);
		Document generateMocesInvalidIssuer = smg.generateMocesInvalidIssuer(domClone);
		Document response = smg.process(generateMocesInvalidIssuer, "mocesInvalidIssuer");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	@Ignore("SDS-6659 We have no expired oces3 certificates yet")
	public void testMocesExpired() {
		Document domClone = smg.domClone(dom4user);
		Document generateMocesExpired = smg.generateMocesExpired(domClone);
		Document response = smg.process(generateMocesExpired, "mocesExpired");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

	@Test
	public void testMocesInvalidSignature() {
		Document domClone = smg.domClone(dom4user);
		Document generateMocesInvalidSignature = smg.generateMocesInvalidSignature(domClone);
		Document response = smg.process(generateMocesInvalidSignature, "mocesInvalidSignature");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_AUTHENTICATION_BAD_ELEMENTS, error.getFaultCode());
		assertEquals(StsFaultActorValues.SEAL, error.getFaultActor());
	}

//	 T5
	@Test
	public void testInvalidCareProviderID() {
		Document domClone = smg.domClone(dom3system);
		Document generateInvalidCareProviderID = smg.generateInvalidCareProviderID(domClone);
		Document response = smg.process(generateInvalidCareProviderID, "invalidCareProviderID");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.STS, error.getFaultActor());
	}

// T6
	/* Removed after the introduction of arbitrary user roles
	@Test
	public void testInvalidRole() {
		Document domClone = smg.domClone(dom4user);
		Document generateInvalidRole = smg.generateInvalidRole(domClone);
		Document response = smg.process(generateInvalidRole, "invalidRole");
		checkDeserializeResponse(response, false);
		factory.deserializeSecurityTokenResponse(XmlUtil.node2String(response));
		WSTError error = smg.getError(response);

		// TODO: in-comment these assertions after STS has been updated.
		// assertNotNull(error);
		// assertEquals(StsFaultCodeValues.WST_REQUEST_FAILED, error.getFaultCode());
		assertEquals(StsFaultActorValues.SEAL, error.getFaultActor());
	}
	*/

/* DISABLED until we know better
	@Test
	public void testInvalidUserAuthorizationCode() {
		Document domClone = RequestGenerator.domClone(dom4user);
		Document generateInvalidUserAuthorizationCode = generateInvalidUserAuthorizationCode(domClone);
		Document response = RequestGenerator.process(generateInvalidUserAuthorizationCode, "invalidUserAuthorizationCode");
		factory.deserializeSecurityTokenResponse(XmlUtil.node2String(response));
		WSTError error = RequestGenerator.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
	}
*/

	@Test
    public void testInvalidUserCivilRegistrationNumber() {
		Document domClone = smg.domClone(dom4user);
		Document generateInvalidUserCivilRegistrationNumber = smg.generateInvalidUserCivilRegistrationNumber(domClone);
		Document response = smg.process(generateInvalidUserCivilRegistrationNumber, "invalidUserCivilRegistrationNumber");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNotNull(error);
		assertEquals(StsFaultCodeValues.WST_FAILED_AUTHENTICATION, error.getFaultCode());
		assertEquals(StsFaultActorValues.UUIDCPR, error.getFaultActor());
	}

    //	 T7
	@Test
	@Ignore("SDS-6659: This had the comment 'remove ignore when STS 2.8.7 is on test1 environment', " +
			"but it fails, even though this version has long since been deployed to test1")
	public void testMocesValidLevel4User() {
		Document domClone = smg.domClone(dom4user);
		Document generateMocesValidLevel4User = smg.generateMocesValidLevel4User(domClone);
		Document response = smg.process(generateMocesValidLevel4User, "mocesValidLevel4User");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNull(error);
	}

	@Test
	public void testVocesValidLevel3System() {
		if (stsMessageVersion == VERSION.OCES3) {
			setUp(voces3Cvr); // because the voces3 test certificate has another cvr
		}
		Document domClone = smg.domClone(dom3system);
		Document generateVocesValidLevel3System = smg.generateVocesValidLevel3System(domClone);
		Document response = smg.process(generateVocesValidLevel3System, "vocesValidLevel3User");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNull(error);
	}

	@Test
	public void testVocesValidLevel3User() {
		if (stsMessageVersion == VERSION.OCES3) {
			setUp(voces3Cvr); // because the voces3 test certificate has another cvr
		}
		Document domClone = smg.domClone(dom3user);
		Document generateVocesValidLevel3User = smg.generateVocesValidLevel3User(domClone);
		Document response = smg.process(generateVocesValidLevel3User, "vocesValidLevel3User");
		checkDeserializeResponse(response, true);
		WSTError error = smg.getError(response);
		assertNull(error);
	}
	
	// ==========================
	
	private void checkDeserializeResponse(Document response, boolean checkMessageID) {
		Message msg = factory.deserializeSecurityTokenResponse(XmlUtil.node2String(response));
		if(checkMessageID &&  (msg.getMessageID()==null || "".equals(msg.getMessageID().trim()))) {
			System.out.println("Warning: deserialized response has empty message ID!");
			Thread.dumpStack();
		}
	}
}